@@ -1,9 +1,10 @@
-/*=========================================================================*\
-* crl.c
-* X509 certificate revoke routines for lua-openssl binding
-*
-* Author:  george zhao <zhaozg(at)gmail.com>
-\*=========================================================================*/
+/***
+x509.crl module for lua-openssl binding, provide x509_crl as lua object.
+create and manage x509 certificate sign request
+@module x509.crl
+@usage
+  crl = require'openssl'.x509.crl
+*/
 #include "openssl.h"
 #include "private.h"
 #define CRYPTO_LOCK_REF
@@ -13,18 +14,33 @@
 int   X509_CRL_cmp(const X509_CRL *a, const X509_CRL *b);
 int   X509_CRL_match(const X509_CRL *a, const X509_CRL *b);
 
+#ifndef CRL_REASON_NONE
+#define CRL_REASON_NONE                         -1;
+#define CRL_REASON_UNSPECIFIED                  0
+#define CRL_REASON_KEY_COMPROMISE               1
+#define CRL_REASON_CA_COMPROMISE                2
+#define CRL_REASON_AFFILIATION_CHANGED          3
+#define CRL_REASON_SUPERSEDED                   4
+#define CRL_REASON_CESSATION_OF_OPERATION       5
+#define CRL_REASON_CERTIFICATE_HOLD             6
+#define CRL_REASON_REMOVE_FROM_CRL              8
+#define CRL_REASON_PRIVILEGE_WITHDRAWN          9
+#define CRL_REASON_AA_COMPROMISE                10
+#endif
+
 static const BIT_STRING_BITNAME reason_flags[] =
 {
-  {0, "Unused", "unused"},
-  {1, "Key Compromise", "keyCompromise"},
-  {2, "CA Compromise", "CACompromise"},
-  {3, "Affiliation Changed", "affiliationChanged"},
-  {4, "Superseded", "superseded"},
-  {5, "Cessation Of Operation", "cessationOfOperation"},
-  {6, "Certificate Hold", "certificateHold"},
-  {7, "Privilege Withdrawn", "privilegeWithdrawn"},
-  {8, "AA Compromise", "AACompromise"},
-  { -1, NULL, NULL}
+  { CRL_REASON_UNSPECIFIED, "Unspecified", "unspecified"},
+  { CRL_REASON_KEY_COMPROMISE,      "Key Compromise", "keyCompromise" },
+  { CRL_REASON_CA_COMPROMISE,       "CA Compromise", "CACompromise" },
+  { CRL_REASON_AFFILIATION_CHANGED, "Affiliation Changed", "affiliationChanged" },
+  { CRL_REASON_SUPERSEDED,          "Superseded", "superseded" },
+  { CRL_REASON_CESSATION_OF_OPERATION, "Cessation Of Operation", "cessationOfOperation" },
+  { CRL_REASON_CERTIFICATE_HOLD,    "Certificate Hold", "certificateHold" },
+  { CRL_REASON_REMOVE_FROM_CRL,     "Remove From CRL", "removeFromCRL" },
+  { CRL_REASON_PRIVILEGE_WITHDRAWN, "Privilege Withdrawn", "privilegeWithdrawn" },
+  { CRL_REASON_AA_COMPROMISE,       "AA Compromise", "AACompromise" },
+  { -1, NULL, NULL }
 };
 
 static const int reason_num = sizeof(reason_flags) / sizeof(BIT_STRING_BITNAME) - 1;
@@ -76,6 +92,16 @@ static int reason_get(lua_State*L, int reasonidx)
   return reason;
 }
 
+static int openssl_x509_revoked_get_reason(X509_REVOKED *revoked)
+{
+  int crit = 0;
+  int reason;
+  ASN1_ENUMERATED *areason = X509_REVOKED_get_ext_d2i(revoked, NID_crl_reason, &crit, NULL);
+  reason = (crit == -1) ? CRL_REASON_NONE : ASN1_ENUMERATED_get(areason);
+  ASN1_ENUMERATED_free(areason);
+  return reason;
+}
+
 static X509_REVOKED *create_revoked(const BIGNUM* bn, time_t t, int reason)
 {
   X509_REVOKED *revoked = X509_REVOKED_new();
@@ -86,9 +112,7 @@ static X509_REVOKED *create_revoked(const BIGNUM* bn, time_t t, int reason)
 
   X509_REVOKED_set_revocationDate(revoked, tm);
   X509_REVOKED_set_serialNumber(revoked, it);
-#if OPENSSL_VERSION_NUMBER > 0x10000000L
-  revoked->reason = reason;
-#else
+
   {
     ASN1_ENUMERATED * e = ASN1_ENUMERATED_new();
     X509_EXTENSION * ext = X509_EXTENSION_new();
@@ -102,57 +126,44 @@ static X509_REVOKED *create_revoked(const BIGNUM* bn, time_t t, int reason)
     X509_EXTENSION_free(ext);
     ASN1_ENUMERATED_free(e);
   }
-#endif
+
   ASN1_TIME_free(tm);
   ASN1_INTEGER_free(it);
 
   return revoked;
 }
 
-static LUA_FUNCTION(openssl_crl_add_revocked)
+static int openssl_revoked2table(lua_State*L, X509_REVOKED *revoked)
 {
-  X509_CRL *crl = CHECK_OBJECT(1, X509_CRL, "openssl.x509_crl");
-  BIGNUM* sn = BN_get(L, 2);
-  time_t t = lua_tointeger(L, 3);
-  int reason = reason_get(L, 4);
+  int reason = openssl_x509_revoked_get_reason(revoked);
+  lua_newtable(L);
+  AUXILIAR_SET(L, -1, "code", reason, number);
+  AUXILIAR_SET(L, -1, "reason", openssl_i2s_revoke_reason(reason), string);
 
-  int ret = 0;
-  X509_REVOKED* revoked = create_revoked(sn, t, reason);
-  ret = X509_CRL_add0_revoked(crl, revoked);
-  lua_pushboolean(L, ret);
-  BN_free(sn);
+  PUSH_ASN1_INTEGER(L, X509_REVOKED_get0_serialNumber(revoked));
+  lua_setfield(L, -2, "serialNumber");
+
+  PUSH_ASN1_TIME(L, X509_REVOKED_get0_revocationDate(revoked));
+  lua_setfield(L, -2, "revocationDate");
+
+  lua_pushstring(L, "extensions");
+  openssl_sk_x509_extension_totable(L, X509_REVOKED_get0_extensions(revoked));
+  lua_rawset(L, -3);
   return 1;
 }
 
-static int openssl_crl_extensions(lua_State* L)
-{
-  X509_CRL *crl = CHECK_OBJECT(1, X509_CRL, "openssl.x509_crl");
-  if (lua_isnone(L, 2))
-  {
-    STACK_OF(X509_EXTENSION) *exts = crl->crl->extensions;
-    if (exts)
-    {
-      openssl_sk_x509_extension_totable(L, exts);
-    }
-    else
-      lua_pushnil(L);
-    return 1;
-  }
-  else
-  {
-    STACK_OF(X509_EXTENSION) *exts = openssl_sk_x509_extension_fromtable(L, 2);
-    int i, n;
-    n = sk_X509_EXTENSION_num(exts);
-    for (i = 0; i < n; i++)
-    {
-      X509_EXTENSION *ext = sk_X509_EXTENSION_value(exts, i);
-      X509_CRL_add_ext(crl, X509_EXTENSION_dup(ext), i);
-    };
-    sk_X509_EXTENSION_pop_free(exts, X509_EXTENSION_free);
-    return openssl_pushresult(L, 1);
-  }
-}
-
+/***
+create or generate a new x509_crl object.
+Note if not give evp_pkey, will create a new x509_crl object,if give will generate a signed x509_crl object.
+@function new
+@tparam[opt] table revoked_list
+@tparam[opt] x509 cacert ca cert to sign x509_crl
+@tparam[opt] evp_pkey capkey private key to sign x509_crl
+@tparam[opt] string|evp_md md_alg
+@tparam[opt=7*24*3600] number period to generate new crl
+@treturn x509_crl object
+@see x509_crl
+*/
 static LUA_FUNCTION(openssl_crl_new)
 {
   int i;
@@ -208,7 +219,7 @@ static LUA_FUNCTION(openssl_crl_new)
       luaL_argcheck(L, X509_check_private_key(cacert, capkey) == 1, 3, "evp_pkey not match with x509 in #2");
     }
   }
-  md = lua_isnoneornil(L, 4) ? EVP_get_digestbyname("sha1") : get_digest(L, 4);
+  md = get_digest(L, 4, "sha256");;
   step = lua_isnoneornil(L, 5) ? 7 * 24 * 3600 : luaL_checkint(L, 5);
 
   if (ret == 1)
@@ -247,6 +258,14 @@ static LUA_FUNCTION(openssl_crl_new)
   return 1;
 }
 
+/***
+read x509_crl from string or bio input
+@function read
+@tparam bio|string input input data
+@tparam[opt='auto'] string format support 'auto','pem','der'
+@treturn x509_crl certificate sign request object
+@see x509_crl
+*/
 static LUA_FUNCTION(openssl_crl_read)
 {
   BIO * in = load_bio_object(L, 1);
@@ -261,12 +280,12 @@ static LUA_FUNCTION(openssl_crl_read)
   if (fmt == FORMAT_PEM)
   {
     crl = PEM_read_bio_X509_CRL(in, NULL, NULL, NULL);
-    BIO_reset(in);
+    (void)BIO_reset(in);
   }
   else if (fmt == FORMAT_DER)
   {
     crl = d2i_X509_CRL_bio(in, NULL);
-    BIO_reset(in);
+    (void)BIO_reset(in);
   }
   BIO_free(in);
   if (crl)
@@ -277,6 +296,44 @@ static LUA_FUNCTION(openssl_crl_read)
   return openssl_pushresult(L, 0);
 }
 
+/***
+list all support reason info
+@function reason
+@treturn table contain support reason node like {lname=...,sname=...,bitnum=...}
+*/
+static int openssl_crl_reason(lua_State *L)
+{
+  int i;
+  const BIT_STRING_BITNAME* bitname;
+  lua_newtable(L);
+  for (i = 0, bitname = &reason_flags[i]; bitname->bitnum != -1; i++, bitname = &reason_flags[i])
+  {
+    openssl_push_bit_string_bitname(L, bitname);
+    lua_rawseti(L, -2, i + 1);
+  }
+  return 1;
+}
+
+static luaL_Reg R[] =
+{
+  {"new",       openssl_crl_new },
+  {"read",      openssl_crl_read},
+  {"reason",    openssl_crl_reason},
+
+  {NULL,    NULL}
+};
+
+/***
+openssl.x509_crl object
+@type x509_crl
+*/
+
+/***
+set version key
+@function version
+@tparam integer version
+@treturn boolean result
+*/
 static LUA_FUNCTION(openssl_crl_version)
 {
   X509_CRL *crl = CHECK_OBJECT(1, X509_CRL, "openssl.x509_crl");
@@ -293,6 +350,80 @@ static LUA_FUNCTION(openssl_crl_version)
   }
 }
 
+/***
+add revoked entry to x509_crl object
+@function add
+@tparam string|number|bn serial
+@tparam number revokedtime
+@tparam[opt=0] number|string reason
+@treturn boolean result true for add success
+*/
+static LUA_FUNCTION(openssl_crl_add_revocked)
+{
+  X509_CRL *crl = CHECK_OBJECT(1, X509_CRL, "openssl.x509_crl");
+  BIGNUM* sn = BN_get(L, 2);
+  time_t t = lua_tointeger(L, 3);
+  int reason = reason_get(L, 4);
+
+  int ret = 0;
+  X509_REVOKED* revoked = create_revoked(sn, t, reason);
+  ret = X509_CRL_add0_revoked(crl, revoked);
+  lua_pushboolean(L, ret);
+  BN_free(sn);
+  return 1;
+}
+
+/***
+get extensions of x509_crl
+@function extensions
+@treturn stack_of_x509_extension extensions
+*/
+/***
+set extensions to x509_crl object
+@function extensions
+@tparam stack_of_x509_extension extensions add to x509_crl
+@treturn boolean result
+*/
+static int openssl_crl_extensions(lua_State* L)
+{
+  X509_CRL *crl = CHECK_OBJECT(1, X509_CRL, "openssl.x509_crl");
+  if (lua_isnone(L, 2))
+  {
+    const STACK_OF(X509_EXTENSION) *exts = X509_CRL_get0_extensions(crl);
+    if (exts)
+    {
+      openssl_sk_x509_extension_totable(L, exts);
+    }
+    else
+      lua_pushnil(L);
+    return 1;
+  }
+  else
+  {
+    STACK_OF(X509_EXTENSION) *exts = (STACK_OF(X509_EXTENSION) *)openssl_sk_x509_extension_fromtable(L, 2);
+    int i, n;
+    n = sk_X509_EXTENSION_num(exts);
+    for (i = 0; i < n; i++)
+    {
+      X509_EXTENSION *ext = sk_X509_EXTENSION_value(exts, i);
+      X509_CRL_add_ext(crl, X509_EXTENSION_dup(ext), i);
+    };
+    sk_X509_EXTENSION_pop_free(exts, X509_EXTENSION_free);
+    return openssl_pushresult(L, 1);
+  }
+}
+
+/***
+get issuer x509_name object
+@function issuer
+@treturn x509_name
+*/
+/***
+set issuer x509_name object
+@function issuer
+@tparam x509_name|x509 issuer
+@treturn boolean result
+*/
 static LUA_FUNCTION(openssl_crl_issuer)
 {
   X509_CRL *crl = CHECK_OBJECT(1, X509_CRL, "openssl.x509_crl");
@@ -300,13 +431,13 @@ static LUA_FUNCTION(openssl_crl_issuer)
   {
     return openssl_push_xname_asobject(L, X509_CRL_get_issuer(crl));
   }
-  else if (auxiliar_isclass(L, "openssl.x509_name", 2))
+  else if (auxiliar_getclassudata(L, "openssl.x509_name", 2))
   {
     X509_NAME* xn = CHECK_OBJECT(2, X509_NAME, "openssl.x509_name");
     int ret = X509_CRL_set_issuer_name(crl, xn);
     return openssl_pushresult(L, ret);
   }
-  else if (auxiliar_isclass(L, "openssl.x509", 2))
+  else if (auxiliar_getclassudata(L, "openssl.x509", 2))
   {
     X509* x = CHECK_OBJECT(2, X509, "openssl.x509");
     int ret = X509_CRL_set_issuer_name(crl, X509_get_issuer_name(x));
@@ -319,6 +450,17 @@ static LUA_FUNCTION(openssl_crl_issuer)
   return 0;
 }
 
+/***
+get lastUpdate time
+@function lastUpdate
+@treturn string lastUpdate
+*/
+/***
+set lastUpdate time
+@function lastUpdate
+@tparam number lastUpdate
+@treturn boolean result
+*/
 static LUA_FUNCTION(openssl_crl_lastUpdate)
 {
   X509_CRL *crl = CHECK_OBJECT(1, X509_CRL, "openssl.x509_crl");
@@ -341,6 +483,17 @@ static LUA_FUNCTION(openssl_crl_lastUpdate)
   }
 }
 
+/***
+get nextUpdate time
+@function nextUpdate
+@treturn string nextUpdate
+*/
+/***
+set nextUpdate time
+@function nextUpdate
+@tparam number nextUpdate
+@treturn boolean result
+*/
 static LUA_FUNCTION(openssl_crl_nextUpdate)
 {
   X509_CRL *crl = CHECK_OBJECT(1, X509_CRL, "openssl.x509_crl");
@@ -363,6 +516,18 @@ static LUA_FUNCTION(openssl_crl_nextUpdate)
   }
 }
 
+/***
+get updateTime time
+@function updateTime
+@treturn string lastUpdate
+*/
+/***
+set updateTime time
+@function updateTime
+@tparam[opt=os.time()] lastUpdate, default use current time
+@tparam number periord periord how long time(seconds)
+@treturn boolean result
+*/
 static LUA_FUNCTION(openssl_crl_updateTime)
 {
   X509_CRL *crl = CHECK_OBJECT(1, X509_CRL, "openssl.x509_crl");
@@ -408,6 +573,11 @@ static LUA_FUNCTION(openssl_crl_updateTime)
   }
 }
 
+/***
+sore crl entry in x509_crl object
+@function sort
+@treturn boolean result true for success and others for fail
+*/
 static LUA_FUNCTION(openssl_crl_sort)
 {
   X509_CRL *crl = CHECK_OBJECT(1, X509_CRL, "openssl.x509_crl");
@@ -415,22 +585,28 @@ static LUA_FUNCTION(openssl_crl_sort)
   return openssl_pushresult(L, ret);
 }
 
+/***
+verify x509_crl object signature
+@function verify
+@tparam x509|evp_pkey key ca cert or public to verify signature
+@treturn boolean result true for success and others for fail
+*/
 static LUA_FUNCTION(openssl_crl_verify)
 {
   X509_CRL *crl = CHECK_OBJECT(1, X509_CRL, "openssl.x509_crl");
   EVP_PKEY *pub = NULL;
   int ret;
-  luaL_argcheck(L, 
-    auxiliar_isclass(L, "openssl.x509", 2) ||
-    auxiliar_isclass(L, "openssl.evp_pkey", 2),
-    2,
-    "must be x509 or evp_pkey object");
-  if (auxiliar_isclass(L, "openssl.evp_pkey", 2))
+  luaL_argcheck(L,
+                auxiliar_getclassudata(L, "openssl.x509", 2) ||
+                auxiliar_getclassudata(L, "openssl.evp_pkey", 2),
+                2,
+                "must be x509 or evp_pkey object");
+  if (auxiliar_getclassudata(L, "openssl.evp_pkey", 2))
   {
     pub = CHECK_OBJECT(2, EVP_PKEY, "openssl.evp_pkey");
     ret = X509_CRL_verify(crl, pub);
   }
-  else 
+  else
   {
     X509* cacert = CHECK_OBJECT(2, X509, "openssl.x509");
     pub = X509_get_pubkey(cacert);
@@ -438,27 +614,32 @@ static LUA_FUNCTION(openssl_crl_verify)
     EVP_PKEY_free(pub);
   }
 
-  
   return openssl_pushresult(L, ret);
 }
 
+/***
+sign x509_crl
+@function sign
+@tparam evp_pkey pkey private key to sign x509
+@tparam x509|x509_name cacert or cacert x509_name
+@tparam[opt='sha1WithRSAEncryption'] string|md_digest md_alg
+@treturn boolean result true for check pass
+*/
 LUA_FUNCTION(openssl_crl_sign)
 {
   X509_CRL *crl = CHECK_OBJECT(1, X509_CRL, "openssl.x509_crl");
   EVP_PKEY *key = CHECK_OBJECT(2, EVP_PKEY, "openssl.evp_pkey");
-  const EVP_MD *md = lua_isnoneornil(L, 4)
-                     ? EVP_get_digestbyname("sha1") : get_digest(L, 4);
-
+  const EVP_MD *md = get_digest(L, 4, "sha256");
   int ret = 1;
 
-  luaL_argcheck(L, auxiliar_isclass(L, "openssl.x509", 3) || auxiliar_isclass(L, "openssl.x509_name", 3),
+  luaL_argcheck(L, auxiliar_getclassudata(L, "openssl.x509", 3) || auxiliar_getclassudata(L, "openssl.x509_name", 3),
                 3, "must be openssl.x509 or openssl.x509_name object");
-  if (auxiliar_isclass(L, "openssl.x509_name", 3))
+  if (auxiliar_getclassudata(L, "openssl.x509_name", 3))
   {
     X509_NAME* xn = CHECK_OBJECT(3, X509_NAME, "openssl.x509_name");
     ret = X509_CRL_set_issuer_name(crl, xn);
   }
-  else if (auxiliar_isclass(L, "openssl.x509", 3))
+  else if (auxiliar_getclassudata(L, "openssl.x509", 3))
   {
     X509* ca = CHECK_OBJECT(3, X509, "openssl.x509");
     ret = X509_CRL_set_issuer_name(crl, X509_get_issuer_name(ca));
@@ -478,13 +659,18 @@ LUA_FUNCTION(openssl_crl_sign)
   return openssl_pushresult(L, ret);
 }
 
+/***
+get digest of x509_crl
+@function digest
+@tparam[opt='SHA1'] evp_md|string md_alg default use sha1
+@treturn string digest result
+*/
 static LUA_FUNCTION(openssl_crl_digest)
 {
   X509_CRL *crl = CHECK_OBJECT(1, X509_CRL, "openssl.x509_crl");
   byte buf[EVP_MAX_MD_SIZE];
   unsigned int lbuf = sizeof(buf);
-  const EVP_MD *md = lua_isnoneornil(L, 2)
-                     ? EVP_get_digestbyname("sha1") : get_digest(L, 2);
+  const EVP_MD *md = get_digest(L, 2, "sha256");
 
   int ret =  X509_CRL_digest(crl, md, buf, &lbuf);
   if (ret == 1)
@@ -495,6 +681,14 @@ static LUA_FUNCTION(openssl_crl_digest)
   return openssl_pushresult(L, ret);
 }
 
+/***
+compare with other x509_crl object
+@function cmp
+@tparam x509_crl other
+@treturn boolean result true for equals or false
+@usage
+  x:cmp(y) == (x==y)
+*/
 static LUA_FUNCTION(openssl_crl_cmp)
 {
   X509_CRL *crl = CHECK_OBJECT(1, X509_CRL, "openssl.x509_crl");
@@ -505,13 +699,21 @@ static LUA_FUNCTION(openssl_crl_cmp)
 }
 
 #if OPENSSL_VERSION_NUMBER >= 0x10100000L && !defined (LIBRESSL_VERSION_NUMBER)
+/***
+make a delta x509_crl object
+@function diff
+@tparam x509_crl newer
+@tparam evp_pkey pkey
+@tparam[opt='sha1'] evp_md|string md_alg
+@tparam[opt=0] integer flags
+@treturn x509_crl delta result x509_crl object
+*/
 static LUA_FUNCTION(openssl_crl_diff)
 {
   X509_CRL *crl = CHECK_OBJECT(1, X509_CRL, "openssl.x509_crl");
   X509_CRL *newer = CHECK_OBJECT(2, X509_CRL, "openssl.x509_crl");
   EVP_PKEY* pkey = CHECK_OBJECT(3, EVP_PKEY, "openssl.evp_pkey");
-  const EVP_MD *md = lua_isnoneornil(L, 4)
-                     ? EVP_get_digestbyname("sha1") : get_digest(L, 4);
+  const EVP_MD *md = get_digest(L, 4, "sha256");
   unsigned int flags = luaL_optinteger(L, 5, 0);
   X509_CRL *diff;
 
@@ -524,6 +726,14 @@ static LUA_FUNCTION(openssl_crl_diff)
     lua_pushnil(L);
   return 1;
 }
+
+/***
+check x509_crl with evp_pkey
+@function check
+@tparam evp_pkey pkey
+@tparam[opt=0] integer flags 
+@treturn boolean result true for pass
+*/
 static LUA_FUNCTION(openssl_crl_check)
 {
   X509_CRL *crl = CHECK_OBJECT(1, X509_CRL, "openssl.x509_crl");
@@ -534,10 +744,17 @@ static LUA_FUNCTION(openssl_crl_check)
 }
 #endif
 
+/***
+parse x509_crl object as table
+@function parse
+@tparam[opt=true] shortname default will use short object name
+@treturn table result
+*/
 static LUA_FUNCTION(openssl_crl_parse)
 {
   X509_CRL *crl = CHECK_OBJECT(1, X509_CRL, "openssl.x509_crl");
   int num, i;
+  const X509_ALGOR *alg;
 
   lua_newtable(L);
   AUXILIAR_SET(L, -1, "version", X509_CRL_get_version(crl), integer);
@@ -550,7 +767,7 @@ static LUA_FUNCTION(openssl_crl_parse)
   }
 
   {
-    const EVP_MD *digest = EVP_get_digestbyname("sha1");
+    const EVP_MD *digest = EVP_get_digestbyname("sha256");
     unsigned char md[EVP_MAX_MD_SIZE];
     unsigned int l = sizeof(md);
 
@@ -572,58 +789,50 @@ static LUA_FUNCTION(openssl_crl_parse)
   PUSH_ASN1_TIME(L, X509_CRL_get_nextUpdate(crl));
   lua_setfield(L, -2, "nextUpdate");
 
-  openssl_push_x509_algor(L, crl->crl->sig_alg);
-  lua_setfield(L, -2, "sig_alg");
-
-#if OPENSSL_VERSION_NUMBER > 0x00909000L
-  if (crl->crl_number)
   {
-    PUSH_ASN1_INTEGER(L, crl->crl_number);
-    lua_setfield(L, -2, "crl_number");
+    const ASN1_BIT_STRING *sig = NULL;
+    const X509_ALGOR *sig_alg = NULL;
+
+    X509_CRL_get0_signature(crl, &sig, &alg);
+    PUSH_OBJECT(sig_alg, "openssl.x509_algor");
+    lua_setfield(L, -2, "sig_alg");
+    PUSH_ASN1_STRING(L, sig);
+    lua_setfield(L, -2, "signature");
   }
-#endif
-  if (crl->crl->extensions)
   {
-    lua_pushstring(L, "extensions");
-    openssl_sk_x509_extension_totable(L, crl->crl->extensions);
-    lua_rawset(L, -3);
-  }
-
-  num = sk_X509_REVOKED_num(crl->crl->revoked);
-  lua_newtable(L);
-  for (i = 0; i < num; i++)
-  {
-    X509_REVOKED *revoked = sk_X509_REVOKED_value(crl->crl->revoked, i);
-    lua_newtable(L);
-
-#if OPENSSL_VERSION_NUMBER > 0x10000000L
-    AUXILIAR_SET(L, -1, "CRLReason", openssl_i2s_revoke_reason(revoked->reason), string);
-#else
+    ASN1_INTEGER *crl_number = X509_CRL_get_ext_d2i(crl, NID_crl_number, NULL, NULL);
+    if (crl_number)
     {
-      int crit = 0;
-      void* reason = X509_REVOKED_get_ext_d2i(revoked, NID_crl_reason, &crit, NULL);
-
-      AUXILIAR_SET(L, -1, "CRLReason", openssl_i2s_revoke_reason(ASN1_ENUMERATED_get(reason)), string);
-      ASN1_ENUMERATED_free(reason);
+      PUSH_ASN1_INTEGER(L, crl_number);
+      lua_setfield(L, -2, "crl_number");
     }
-#endif
-    PUSH_ASN1_INTEGER(L, revoked->serialNumber);
-    lua_setfield(L, -2, "serialNumber");
-
-    PUSH_ASN1_TIME(L, revoked->revocationDate);
-    lua_setfield(L, -2, "revocationDate");
-
-    if (crl->crl->extensions)
+  }
+  {
+    const STACK_OF(X509_EXTENSION) *extensions = X509_CRL_get0_extensions(crl);
+    if (extensions)
     {
-      lua_pushstring(L, "extensions");
-      openssl_sk_x509_extension_totable(L, crl->crl->extensions);
-      lua_rawset(L, -3);
+      openssl_sk_x509_extension_totable(L, extensions);
+      lua_setfield(L, -2, "extensions");
+    }
+  }
+
+  {
+    STACK_OF(X509_REVOKED) *revokeds = X509_CRL_get_REVOKED(crl);
+    if (revokeds)
+    {
+      num = sk_X509_REVOKED_num(revokeds);
+      lua_newtable(L);
+      for (i = 0; i < num; i++)
+      {
+        X509_REVOKED *revoked = sk_X509_REVOKED_value(revokeds, i);
+        openssl_revoked2table(L, revoked);
+        lua_rawseti(L, -2, i + 1);
+      }
+
+      lua_setfield(L, -2, "revoked");
     }
-
-    lua_rawseti(L, -2, i + 1);
   }
 
-  lua_setfield(L, -2, "revoked");
   return 1;
 }
 
@@ -634,6 +843,13 @@ static LUA_FUNCTION(openssl_crl_free)
   return 0;
 }
 
+/***
+export x509_crl to string
+@function export
+@tparam[opt='pem'] string format
+@tparam[opt='true'] boolean noext not export extension
+@treturn string
+*/
 static LUA_FUNCTION(openssl_crl_export)
 {
   X509_CRL * crl = CHECK_OBJECT(1, X509_CRL, "openssl.x509_crl");
@@ -677,34 +893,48 @@ static LUA_FUNCTION(openssl_crl_export)
   return 1;
 }
 
-
+/***
+get count of revoked entry
+@function count
+@treturn number count
+@usage
+  assert(#crl==crl:count())
+*/
 static LUA_FUNCTION(openssl_crl_count)
 {
   X509_CRL * crl = CHECK_OBJECT(1, X509_CRL, "openssl.x509_crl");
-  int n = sk_X509_REVOKED_num(crl->crl->revoked);
+  STACK_OF(X509_REVOKED) *revokeds = X509_CRL_get_REVOKED(crl);
+  int n = revokeds ? sk_X509_REVOKED_num(revokeds) : 0;
   lua_pushinteger(L, n);
   return 1;
 }
 
+/***
+get revoekd entry
+@function get
+@tparam number index
+@treturn table revoekd 
+*/
 static LUA_FUNCTION(openssl_crl_get)
 {
   X509_CRL * crl = CHECK_OBJECT(1, X509_CRL, "openssl.x509_crl");
-  int i = 0;
+  STACK_OF(X509_REVOKED) *revokeds = X509_CRL_get_REVOKED(crl);
   X509_REVOKED *revoked = NULL;
+  int i = 0;
   if (lua_isinteger(L, 2))
   {
     i = lua_tointeger(L, 2);
-    luaL_argcheck(L, (i >= 0 && i < sk_X509_REVOKED_num(crl->crl->revoked)), 2, "Out of range");
-    revoked = sk_X509_REVOKED_value(crl->crl->revoked, i);
+    luaL_argcheck(L, (i >= 0 && i < sk_X509_REVOKED_num(revokeds)), 2, "Out of range");
+    revoked = sk_X509_REVOKED_value(revokeds, i);
   }
   else
   {
     ASN1_STRING *sn = CHECK_OBJECT(2, ASN1_STRING, "openssl.asn1_integer");
-    int cnt = sk_X509_REVOKED_num(crl->crl->revoked);
+    int cnt = sk_X509_REVOKED_num(revokeds);
     for (i = 0; i < cnt; i++)
     {
-      X509_REVOKED *rev = sk_X509_REVOKED_value(crl->crl->revoked, i);
-      if (ASN1_STRING_cmp(rev->serialNumber, sn) == 0)
+      X509_REVOKED *rev = sk_X509_REVOKED_value(revokeds, i);
+      if (ASN1_STRING_cmp(X509_REVOKED_get0_serialNumber(rev), sn) == 0)
       {
         revoked = rev;
         break;
@@ -713,32 +943,7 @@ static LUA_FUNCTION(openssl_crl_get)
   }
   if (revoked)
   {
-    lua_newtable(L);
-
-#if OPENSSL_VERSION_NUMBER > 0x10000000L
-    AUXILIAR_SET(L, -1, "code", revoked->reason, number);
-    AUXILIAR_SET(L, -1, "reason", openssl_i2s_revoke_reason(revoked->reason), string);
-#else
-    {
-      int crit = 0;
-      void* reason = X509_REVOKED_get_ext_d2i(revoked, NID_crl_reason, &crit, NULL);
-      AUXILIAR_SET(L, -1, "code", ASN1_ENUMERATED_get(reason), number);
-      AUXILIAR_SET(L, -1, "reason", openssl_i2s_revoke_reason(ASN1_ENUMERATED_get(reason)), string);
-      ASN1_ENUMERATED_free(reason);
-    }
-#endif
-    PUSH_ASN1_INTEGER(L, revoked->serialNumber);
-    lua_setfield(L, -2, "serialNumber");
-
-    PUSH_ASN1_TIME(L, revoked->revocationDate);
-    lua_setfield(L, -2, "revocationDate");
-
-    if (crl->crl->extensions)
-    {
-      lua_pushstring(L, "extensions");
-      openssl_sk_x509_extension_totable(L, crl->crl->extensions);
-      lua_rawset(L, -3);
-    }
+    openssl_revoked2table(L, revoked);
   }
   else
     lua_pushnil(L);
@@ -785,55 +990,38 @@ static luaL_Reg crl_funcs[] =
 static int openssl_revoked_info(lua_State* L)
 {
   X509_REVOKED* revoked = CHECK_OBJECT(1, X509_REVOKED, "openssl.x509_revoked");
-  lua_newtable(L);
-
-#if OPENSSL_VERSION_NUMBER > 0x10000000L
-  AUXILIAR_SET(L, -1, "reason", openssl_i2s_revoke_reason(revoked->reason), string);
-#else
-  {
-    int crit = 0;
-    void* reason = X509_REVOKED_get_ext_d2i(revoked, NID_crl_reason, &crit, NULL);
-
-    AUXILIAR_SET(L, -1, "reason", openssl_i2s_revoke_reason(ASN1_ENUMERATED_get(reason)), string);
-    ASN1_ENUMERATED_free(reason);
-  }
-#endif
-  PUSH_ASN1_INTEGER(L, revoked->serialNumber);
-  lua_setfield(L, -2, "serialNumber");
-
-  PUSH_ASN1_TIME(L, revoked->revocationDate);
-  lua_setfield(L, -2, "revocationDate");
-
-  if (revoked->extensions)
-  {
-    lua_pushstring(L, "extensions");
-    openssl_sk_x509_extension_totable(L, revoked->extensions);
-    lua_rawset(L, -3);
-  }
-  return 1;
+  return openssl_revoked2table(L, revoked);
 };
 
 static int openssl_revoked_reason(lua_State* L)
 {
   X509_REVOKED* revoked = CHECK_OBJECT(1, X509_REVOKED, "openssl.x509_revoked");
-#if OPENSSL_VERSION_NUMBER > 0x00909000L
-  lua_pushstring(L, openssl_i2s_revoke_reason(revoked->reason));
-  lua_pushinteger(L, revoked->reason);
-  return 2;
-#else
-  /*
+  if (lua_isnone(L, 2))
   {
-    int crit = 0;
-    void* reason = X509_REVOKED_get_ext_d2i(revoked, NID_crl_reason, &crit, NULL);
-    lua_pushstring(L, openssl_i2s_revoke_reason(ASN1_ENUMERATED_get(reason)).lname);
-    lua_pushinteger(revoked->reason);
-    ASN1_ENUMERATED_free(reason);
-  }*/
+    int reason = openssl_x509_revoked_get_reason(revoked);
+    lua_pushinteger(L, reason);
+    lua_pushstring(L, openssl_i2s_revoke_reason(reason));
+    return 2;
+  }
+  else
+  {
+    int reason = reason_get(L, 2);
+    ASN1_ENUMERATED * e = ASN1_ENUMERATED_new();
+    X509_EXTENSION * ext = X509_EXTENSION_new();
+
+    ASN1_ENUMERATED_set(e, reason);
+
+    X509_EXTENSION_set_data(ext, e);
+    X509_EXTENSION_set_object(ext, OBJ_nid2obj(NID_crl_reason));
+    X509_REVOKED_add_ext(revoked, ext, 0);
+
+    X509_EXTENSION_free(ext);
+    ASN1_ENUMERATED_free(e);
+  }
   return 0;
-#endif
 }
 
-static time_t ASN1_GetTimeT(ASN1_TIME* time)
+static time_t ASN1_GetTimeT(const ASN1_TIME* time)
 {
   struct tm t;
   const char* str = (const char*) time->data;
@@ -874,27 +1062,29 @@ static time_t ASN1_GetTimeT(ASN1_TIME* time)
 static int openssl_revoked_revocationDate(lua_State* L)
 {
   X509_REVOKED* revoked = CHECK_OBJECT(1, X509_REVOKED, "openssl.x509_revoked");
-  PUSH_ASN1_TIME(L, revoked->revocationDate);
-  lua_pushinteger(L, (LUA_INTEGER)ASN1_GetTimeT(revoked->revocationDate));
+  const ASN1_TIME* time = X509_REVOKED_get0_revocationDate(revoked);
+  lua_pushinteger(L, (LUA_INTEGER)ASN1_GetTimeT(time));
+  PUSH_ASN1_TIME(L, time);
   return 2;
 }
 
 static int openssl_revoked_serialNumber(lua_State* L)
 {
   X509_REVOKED* revoked = CHECK_OBJECT(1, X509_REVOKED, "openssl.x509_revoked");
-  BIGNUM *bn = ASN1_INTEGER_to_BN(revoked->serialNumber, NULL);
-  PUSH_ASN1_INTEGER(L, revoked->serialNumber);
+  const ASN1_INTEGER *serialNumber = X509_REVOKED_get0_serialNumber(revoked);
+  BIGNUM *bn = ASN1_INTEGER_to_BN(serialNumber, NULL);
   PUSH_OBJECT(bn, "openssl.bn");
+  PUSH_ASN1_INTEGER(L, serialNumber);
   return 2;
 }
 
 static int openssl_revoked_extensions(lua_State* L)
 {
   X509_REVOKED* revoked = CHECK_OBJECT(1, X509_REVOKED, "openssl.x509_revoked");
-
-  if (revoked->extensions)
+  const STACK_OF(X509_EXTENSION) *exts = X509_REVOKED_get0_extensions(revoked);
+  if (exts)
   {
-    openssl_sk_x509_extension_totable(L, revoked->extensions);
+    openssl_sk_x509_extension_totable(L, exts);
   }
   else
     lua_pushnil(L);
@@ -922,27 +1112,6 @@ static luaL_Reg revoked_funcs[] =
   {NULL,    NULL}
 };
 
-static int openssl_crl_reason(lua_State *L)
-{
-  int i;
-  const BIT_STRING_BITNAME* bitname;
-  lua_newtable(L);
-  for (i = 0, bitname = &reason_flags[i]; bitname->bitnum != -1; i++, bitname = &reason_flags[i])
-  {
-    openssl_push_bit_string_bitname(L, bitname);
-    lua_rawseti(L, -2, i + 1);
-  }
-  return 1;
-}
-
-static luaL_Reg R[] =
-{
-  {"new",       openssl_crl_new },
-  {"read",      openssl_crl_read},
-  {"reason",    openssl_crl_reason},
-  {NULL,    NULL}
-};
-
 IMP_LUA_SK(X509_CRL, x509_crl)
 
 int luaopen_x509_crl(lua_State *L)
