@@ -1,10 +1,10 @@
-/*=========================================================================*\
-* pkcs7.c
-* PKCS7 module for lua-openssl binding
-*
-* Author:  george zhao <zhaozg(at)gmail.com>
-\*=========================================================================*/
+/***
+PKCS7 module for lua-openssl binding
 
+@module pkcs7
+@usage
+  pkcs7 = require('openssl').pkcs7
+*/
 #include "openssl.h"
 #include "private.h"
 
@@ -12,6 +12,16 @@
 #define MYVERSION MYNAME " library for " LUA_VERSION " / Nov 2014 / "\
   "based on OpenSSL " SHLIB_VERSION_NUMBER
 
+/***
+read string or bio object, which include pkcs7 content
+
+@function read
+@tparam bio|string input
+@tparam[opt='auto'] format allow 'auto','der','pem','smime'
+ auto will only try 'der' or 'pem'
+@treturn pkcs7 object or nil
+@treturn string content exist only smime format
+*/
 static LUA_FUNCTION(openssl_pkcs7_read)
 {
   BIO* bio = load_bio_object(L, 1);
@@ -27,12 +37,12 @@ static LUA_FUNCTION(openssl_pkcs7_read)
   if (fmt == FORMAT_DER)
   {
     p7 = d2i_PKCS7_bio(bio, NULL);
-    BIO_reset(bio);
+    (void)BIO_reset(bio);
   }
   else if (fmt == FORMAT_PEM)
   {
     p7 = PEM_read_bio_PKCS7(bio, NULL, NULL, NULL);
-    BIO_reset(bio);
+    (void)BIO_reset(bio);
   }
   else if (fmt == FORMAT_SMIME)
   {
@@ -57,7 +67,14 @@ static LUA_FUNCTION(openssl_pkcs7_read)
 }
 
 #if OPENSSL_VERSION_NUMBER > 0x10000000L
+/***
+create new empty pkcs7 object, which support flexble sign methods.
 
+@function new
+@tparam[opt=NID_pkcs7_signed] int oid given pkcs7 type
+@tparam[opt=NID_pkcs7_data] int content given pkcs7 content type
+@treturn pkcs7 object
+*/
 static LUA_FUNCTION(openssl_pkcs7_new)
 {
   int type = luaL_optint(L, 1, NID_pkcs7_signed);
@@ -81,23 +98,6 @@ static LUA_FUNCTION(openssl_pkcs7_new)
   return 0;
 }
 
-static LUA_FUNCTION(openssl_pkcs7_sign_add_signer)
-{
-  PKCS7 *p7 = CHECK_OBJECT(1, PKCS7, "openssl.pkcs7");
-  X509 *signcert = CHECK_OBJECT(2, X509, "openssl.x509");
-  EVP_PKEY *pkey = CHECK_OBJECT(3, EVP_PKEY, "openssl.evp_pkey");
-  const EVP_MD* md = get_digest(L, 4);
-  long flags = luaL_optint(L, 5, 0);
-  PKCS7_SIGNER_INFO *signer = 0;
-
-  luaL_argcheck(L, X509_check_private_key(signcert, pkey), 3,
-                "sigcert and private key not match");
-
-  signer = PKCS7_sign_add_signer(p7, signcert, pkey, md, flags);
-  (void) signer;
-  return openssl_pushresult(L, signcert != NULL ? 1 : 0);
-}
-
 static LUA_FUNCTION(openssl_pkcs7_add)
 {
   PKCS7 *p7 = CHECK_OBJECT(1, PKCS7, "openssl.pkcs7");
@@ -107,10 +107,10 @@ static LUA_FUNCTION(openssl_pkcs7_add)
   luaL_argcheck(L, lua_isuserdata(L, 2), 2, "must supply certificate or crl object");
   for (i = 2; i <= n; i++)
   {
-    luaL_argcheck(L, auxiliar_isclass(L, "openssl.x509", i) || auxiliar_isclass(L, "openssl.x509_crl", i),
+    luaL_argcheck(L, auxiliar_getclassudata(L, "openssl.x509", i) || auxiliar_getclassudata(L, "openssl.x509_crl", i),
                   i, "must supply certificate or crl object");
 
-    if (auxiliar_isclass(L, "openssl.x509", i))
+    if (auxiliar_getclassudata(L, "openssl.x509", i))
     {
       X509* x = CHECK_OBJECT(i, X509, "openssl.x509");
       ret = PKCS7_add_certificate(p7, x);
@@ -179,12 +179,11 @@ static BIO *PKCS7_find_digest(EVP_MD_CTX **pmd, BIO *bio, int nid)
       return bio;
     bio = BIO_next(bio);
   }
-  return NULL;
 }
 
 static int PKCS7_SIGNER_INFO_sign_0(PKCS7_SIGNER_INFO *si)
 {
-  EVP_MD_CTX mctx;
+  EVP_MD_CTX *mctx;
   EVP_PKEY_CTX *pctx;
   unsigned char *abuf = NULL;
   int alen;
@@ -195,8 +194,9 @@ static int PKCS7_SIGNER_INFO_sign_0(PKCS7_SIGNER_INFO *si)
   if (md == NULL)
     return 0;
 
-  EVP_MD_CTX_init(&mctx);
-  if (EVP_DigestSignInit(&mctx, &pctx, md, NULL, si->pkey) <= 0)
+  mctx = EVP_MD_CTX_new();
+  EVP_MD_CTX_init(mctx);
+  if (EVP_DigestSignInit(mctx, &pctx, md, NULL, si->pkey) <= 0)
     goto err;
 
   if (EVP_PKEY_CTX_ctrl(pctx, -1, EVP_PKEY_OP_SIGN,
@@ -210,17 +210,17 @@ static int PKCS7_SIGNER_INFO_sign_0(PKCS7_SIGNER_INFO *si)
                        ASN1_ITEM_rptr(PKCS7_ATTR_SIGN));
   if (!abuf)
     goto err;
-  if (EVP_DigestSignUpdate(&mctx, abuf, alen) <= 0)
+  if (EVP_DigestSignUpdate(mctx, abuf, alen) <= 0)
     goto err;
   OPENSSL_free(abuf);
   abuf = NULL;
-  if (EVP_DigestSignFinal(&mctx, NULL, &siglen) <= 0)
+  if (EVP_DigestSignFinal(mctx, NULL, &siglen) <= 0)
     goto err;
   abuf = OPENSSL_malloc(siglen);
   if (!abuf)
     goto err;
 
-  if (EVP_DigestSignFinal(&mctx, abuf, &siglen) <= 0)
+  if (EVP_DigestSignFinal(mctx, abuf, &siglen) <= 0)
     goto err;
   if (EVP_PKEY_CTX_ctrl(pctx, -1, EVP_PKEY_OP_SIGN,
                         EVP_PKEY_CTRL_PKCS7_SIGN, 1, si) <= 0)
@@ -229,7 +229,7 @@ static int PKCS7_SIGNER_INFO_sign_0(PKCS7_SIGNER_INFO *si)
     goto err;
   }
 
-  EVP_MD_CTX_cleanup(&mctx);
+  EVP_MD_CTX_free(mctx);
 
   ASN1_STRING_set0(si->enc_digest, abuf, siglen);
 
@@ -238,7 +238,7 @@ static int PKCS7_SIGNER_INFO_sign_0(PKCS7_SIGNER_INFO *si)
 err:
   if (abuf)
     OPENSSL_free(abuf);
-  EVP_MD_CTX_cleanup(&mctx);
+  EVP_MD_CTX_free(mctx);
   return 0;
 
 }
@@ -314,7 +314,7 @@ static int openssl_pkcs7_dataFinal(PKCS7 *p7, BIO *bio)
   int i, j;
   BIO *btmp;
   PKCS7_SIGNER_INFO *si;
-  EVP_MD_CTX *mdc, ctx_tmp;
+  EVP_MD_CTX *mdc, *ctx_tmp;
   STACK_OF(X509_ATTRIBUTE) *sk;
   STACK_OF(PKCS7_SIGNER_INFO) *si_sk = NULL;
   ASN1_OCTET_STRING *os = NULL;
@@ -330,8 +330,8 @@ static int openssl_pkcs7_dataFinal(PKCS7 *p7, BIO *bio)
     PKCS7err(PKCS7_F_PKCS7_DATAFINAL, PKCS7_R_NO_CONTENT);
     return 0;
   }
-
-  EVP_MD_CTX_init(&ctx_tmp);
+  ctx_tmp = EVP_MD_CTX_new();
+  EVP_MD_CTX_init(ctx_tmp);
   i = OBJ_obj2nid(p7->type);
   p7->state = PKCS7_S_HEADER;
 
@@ -346,7 +346,11 @@ static int openssl_pkcs7_dataFinal(PKCS7 *p7, BIO *bio)
     os = p7->d.signed_and_enveloped->enc_data->enc_data;
     if (!os)
     {
+#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
       os = M_ASN1_OCTET_STRING_new();
+#else
+      os = ASN1_OCTET_STRING_new();
+#endif
       if (!os)
       {
         PKCS7err(PKCS7_F_PKCS7_DATAFINAL, ERR_R_MALLOC_FAILURE);
@@ -360,7 +364,11 @@ static int openssl_pkcs7_dataFinal(PKCS7 *p7, BIO *bio)
     os = p7->d.enveloped->enc_data->enc_data;
     if (!os)
     {
+#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
       os = M_ASN1_OCTET_STRING_new();
+#else
+      os = ASN1_OCTET_STRING_new();
+#endif
       if (!os)
       {
         PKCS7err(PKCS7_F_PKCS7_DATAFINAL, ERR_R_MALLOC_FAILURE);
@@ -375,7 +383,11 @@ static int openssl_pkcs7_dataFinal(PKCS7 *p7, BIO *bio)
     /* If detached data then the content is excluded */
     if (PKCS7_type_is_data(p7->d.sign->contents) && p7->detached)
     {
+#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
       M_ASN1_OCTET_STRING_free(os);
+#else
+      ASN1_OCTET_STRING_free(os);
+#endif
       os = NULL;
       p7->d.sign->contents->d.data = NULL;
     }
@@ -386,7 +398,11 @@ static int openssl_pkcs7_dataFinal(PKCS7 *p7, BIO *bio)
     /* If detached data then the content is excluded */
     if (PKCS7_type_is_data(p7->d.digest->contents) && p7->detached)
     {
+#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
       M_ASN1_OCTET_STRING_free(os);
+#else
+      ASN1_OCTET_STRING_free(os);
+#endif
       os = NULL;
       p7->d.digest->contents->d.data = NULL;
     }
@@ -417,7 +433,7 @@ static int openssl_pkcs7_dataFinal(PKCS7 *p7, BIO *bio)
       /*
       * We now have the EVP_MD_CTX, lets do the signing.
       */
-      if (!EVP_MD_CTX_copy_ex(&ctx_tmp, mdc))
+      if (!EVP_MD_CTX_copy_ex(ctx_tmp, mdc))
         goto err;
 
       sk = si->auth_attr;
@@ -428,7 +444,7 @@ static int openssl_pkcs7_dataFinal(PKCS7 *p7, BIO *bio)
       */
       if (sk_X509_ATTRIBUTE_num(sk) > 0)
       {
-        if (!do_pkcs7_signed_attrib(si, &ctx_tmp))
+        if (!do_pkcs7_signed_attrib(si, ctx_tmp))
           goto err;
       }
       else
@@ -440,7 +456,7 @@ static int openssl_pkcs7_dataFinal(PKCS7 *p7, BIO *bio)
         if (!abuf)
           goto err;
 
-        if (!EVP_SignFinal(&ctx_tmp, abuf, &abuflen, si->pkey))
+        if (!EVP_SignFinal(ctx_tmp, abuf, &abuflen, si->pkey))
         {
           PKCS7err(PKCS7_F_PKCS7_DATAFINAL, ERR_R_EVP_LIB);
           goto err;
@@ -458,7 +474,11 @@ static int openssl_pkcs7_dataFinal(PKCS7 *p7, BIO *bio)
       goto err;
     if (!EVP_DigestFinal_ex(mdc, md_data, &md_len))
       goto err;
+#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
     M_ASN1_OCTET_STRING_set(p7->d.digest->digest, md_data, md_len);
+#else
+    ASN1_OCTET_STRING_set(p7->d.digest->digest, md_data, md_len);
+#endif
   }
 
   if (!PKCS7_is_detached(p7))
@@ -491,7 +511,7 @@ static int openssl_pkcs7_dataFinal(PKCS7 *p7, BIO *bio)
   }
   ret = 1;
 err:
-  EVP_MD_CTX_cleanup(&ctx_tmp);
+  EVP_MD_CTX_free(ctx_tmp);
   return (ret);
 }
 
@@ -534,210 +554,21 @@ static int openssl_pkcs7_final(lua_State *L)
   return ret;
 }
 
-
-static LUA_FUNCTION(openssl_pkcs7_sign_digest)
-{
-  PKCS7 *p7 = CHECK_OBJECT(1, PKCS7, "openssl.pkcs7");
-  size_t l;
-  const char* data = luaL_checklstring(L, 2, &l);
-  long flags = luaL_optint(L, 3, 0);
-  int hash = lua_isnoneornil(L, 4) ? 0 : lua_toboolean(L, 4);
-
-  int ret = 0;
-  int i, j;
-
-  const EVP_MD* md;
-  PKCS7_SIGNER_INFO *si;
-  EVP_MD_CTX mdc;
-  STACK_OF(X509_ATTRIBUTE) *sk;
-  STACK_OF(PKCS7_SIGNER_INFO) *si_sk = NULL;
-  ASN1_OCTET_STRING *os = NULL;
-
-  if (p7->d.ptr == NULL)
-  {
-    luaL_error(L, "pkcs7 without content");
-    return 0;
-  }
-
-  flags |= PKCS7_DETACHED;
-  PKCS7_set_detached(p7, 1);
-
-  EVP_MD_CTX_init(&mdc);
-  i = OBJ_obj2nid(p7->type);
-  p7->state = PKCS7_S_HEADER;
-
-  switch (i)
-  {
-  case NID_pkcs7_data:
-    os = p7->d.data;
-    break;
-  case NID_pkcs7_signedAndEnveloped:
-    /* XXXXXXXXXXXXXXXX */
-    si_sk = p7->d.signed_and_enveloped->signer_info;
-    os = p7->d.signed_and_enveloped->enc_data->enc_data;
-    if (!os)
-    {
-      os = M_ASN1_OCTET_STRING_new();
-      if (!os)
-      {
-        PKCS7err(PKCS7_F_PKCS7_DATAFINAL, ERR_R_MALLOC_FAILURE);
-        goto err;
-      }
-      p7->d.signed_and_enveloped->enc_data->enc_data = os;
-    }
-    break;
-  case NID_pkcs7_enveloped:
-    /* XXXXXXXXXXXXXXXX */
-    os = p7->d.enveloped->enc_data->enc_data;
-    if (!os)
-    {
-      os = M_ASN1_OCTET_STRING_new();
-      if (!os)
-      {
-        PKCS7err(PKCS7_F_PKCS7_DATAFINAL, ERR_R_MALLOC_FAILURE);
-        goto err;
-      }
-      p7->d.enveloped->enc_data->enc_data = os;
-    }
-    break;
-  case NID_pkcs7_signed:
-    si_sk = p7->d.sign->signer_info;
-    os = PKCS7_get_octet_string(p7->d.sign->contents);
-    /* If detached data then the content is excluded */
-    if (PKCS7_type_is_data(p7->d.sign->contents) && p7->detached)
-    {
-      M_ASN1_OCTET_STRING_free(os);
-      os = NULL;
-      p7->d.sign->contents->d.data = NULL;
-    }
-    break;
-
-  case NID_pkcs7_digest:
-    os = PKCS7_get_octet_string(p7->d.digest->contents);
-    /* If detached data then the content is excluded */
-    if (PKCS7_type_is_data(p7->d.digest->contents) && p7->detached)
-    {
-      M_ASN1_OCTET_STRING_free(os);
-      os = NULL;
-      p7->d.digest->contents->d.data = NULL;
-    }
-    break;
-
-  default:
-    PKCS7err(PKCS7_F_PKCS7_DATAFINAL, PKCS7_R_UNSUPPORTED_CONTENT_TYPE);
-    goto err;
-  }
-
-  if (si_sk != NULL)
-  {
-    for (i = 0; i < sk_PKCS7_SIGNER_INFO_num(si_sk); i++)
-    {
-      si = sk_PKCS7_SIGNER_INFO_value(si_sk, i);
-      if (si->pkey == NULL)
-        continue;
-      j = OBJ_obj2nid(si->digest_alg->algorithm);
-      md = EVP_get_digestbynid(j);
-      EVP_DigestInit_ex(&mdc, md, NULL);
-
-      if (hash)
-      {
-        if (l == (size_t) mdc.digest->ctx_size)
-        {
-          memcpy(mdc.md_data, data, l);
-        }
-        else
-        {
-          EVP_MD_CTX_cleanup(&mdc);
-          luaL_argerror(L, 2, "data with wrong length");
-        }
-      }
-      else
-        EVP_DigestUpdate(&mdc, data, l);
-
-      sk = si->auth_attr;
-
-      /*
-      * If there are attributes, we add the digest attribute and only
-      * sign the attributes
-      */
-      if (sk_X509_ATTRIBUTE_num(sk) > 0)
-      {
-        if (!do_pkcs7_signed_attrib(si, &mdc))
-          goto err;
-      }
-      else
-      {
-        unsigned char *abuf = NULL;
-        unsigned int abuflen;
-        abuflen = EVP_PKEY_size(si->pkey);
-        abuf = OPENSSL_malloc(abuflen);
-        if (!abuf)
-          goto err;
-
-        if (!EVP_SignFinal(&mdc, abuf, &abuflen, si->pkey))
-        {
-          PKCS7err(PKCS7_F_PKCS7_DATAFINAL, ERR_R_EVP_LIB);
-          goto err;
-        }
-        ASN1_STRING_set0(si->enc_digest, abuf, abuflen);
-      }
-    }
-  }
-  else if (i == NID_pkcs7_digest)
-  {
-    unsigned char md_data[EVP_MAX_MD_SIZE];
-    unsigned int md_len;
-    md = EVP_get_digestbynid(OBJ_obj2nid(p7->d.digest->md->algorithm));
-    EVP_DigestInit_ex(&mdc, md, NULL);
-    if (l == (size_t) mdc.digest->ctx_size)
-    {
-      memcpy(mdc.md_data, data, l);
-    }
-    else
-    {
-      EVP_MD_CTX_cleanup(&mdc);
-      luaL_error(L, "data with wrong data");
-    }
-    if (!EVP_DigestFinal_ex(&mdc, md_data, &md_len))
-      goto err;
-    M_ASN1_OCTET_STRING_set(p7->d.digest->digest, md_data, md_len);
-  }
-
-  if (!PKCS7_is_detached(p7))
-  {
-    /*
-    * NOTE(emilia): I think we only reach os == NULL here because detached
-    * digested data support is broken.
-    */
-    if (os == NULL)
-      goto err;
-    if (!(os->flags & ASN1_STRING_FLAG_NDEF))
-    {
-      char *cont = memdup(data, l);
-      long contlen = l;
-      ASN1_STRING_set0(os, (unsigned char *) cont, contlen);
-    }
-  }
-
-  ret = 1;
-err:
-  EVP_MD_CTX_cleanup(&mdc);
-  return openssl_pushresult(L, ret);
-}
-
 int PKCS7_signatureVerify_digest(PKCS7 *p7, PKCS7_SIGNER_INFO *si, X509 *x509,
                                  const unsigned char* data, size_t len, int hash)
 {
   ASN1_OCTET_STRING *os;
   const EVP_MD* md;
-  EVP_MD_CTX mdc, mdc_tmp;
+  EVP_MD_CTX *mdc, *mdc_tmp;
   int ret = 0, i;
   int md_type;
   STACK_OF(X509_ATTRIBUTE) *sk;
   EVP_PKEY *pkey = NULL;
 
-  EVP_MD_CTX_init(&mdc);
-  EVP_MD_CTX_init(&mdc_tmp);
+  mdc = EVP_MD_CTX_new();
+  mdc_tmp = EVP_MD_CTX_new();
+  EVP_MD_CTX_init(mdc);
+  EVP_MD_CTX_init(mdc_tmp);
   if (!PKCS7_type_is_signed(p7) && !PKCS7_type_is_signedAndEnveloped(p7))
   {
     PKCS7err(PKCS7_F_PKCS7_SIGNATUREVERIFY, PKCS7_R_WRONG_PKCS7_TYPE);
@@ -746,15 +577,27 @@ int PKCS7_signatureVerify_digest(PKCS7 *p7, PKCS7_SIGNER_INFO *si, X509 *x509,
 
   md_type = OBJ_obj2nid(si->digest_alg->algorithm);
   md = EVP_get_digestbynid(md_type);
+#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
   if (!md || !data || (hash && len != (size_t) md->ctx_size) )
     goto err;
 
-  if (!EVP_DigestInit_ex(&mdc, md, NULL))
+  if (!EVP_DigestInit_ex(mdc, md, NULL))
     goto err;
   if (hash)
-    memcpy(mdc.md_data, data, len);
+    memcpy(mdc->md_data, data, len);
   else
-    EVP_DigestUpdate(&mdc, data, len);
+    EVP_DigestUpdate(mdc, data, len);
+#else
+  if (!md || !data || (hash && len != (size_t)EVP_MD_meth_get_app_datasize(md)) )
+    goto err;
+
+  if (!EVP_DigestInit_ex(mdc, md, NULL))
+    goto err;
+  if (hash)
+    memcpy(EVP_MD_CTX_md_data(mdc), data, len);
+  else
+    EVP_DigestUpdate(mdc, data, len);
+#endif
 
   pkey = X509_get_pubkey(x509);
   if (!pkey)
@@ -766,7 +609,7 @@ int PKCS7_signatureVerify_digest(PKCS7 *p7, PKCS7_SIGNER_INFO *si, X509 *x509,
   * mdc is the digest ctx that we want, unless there are attributes, in
   * which case the digest is the signed attributes
   */
-  if (!EVP_MD_CTX_copy_ex(&mdc_tmp, &mdc))
+  if (!EVP_MD_CTX_copy_ex(mdc_tmp, mdc))
     goto err;
   sk = si->auth_attr;
   if ((sk != NULL) && (sk_X509_ATTRIBUTE_num(sk) != 0))
@@ -776,7 +619,7 @@ int PKCS7_signatureVerify_digest(PKCS7 *p7, PKCS7_SIGNER_INFO *si, X509 *x509,
     int alen;
     ASN1_OCTET_STRING *message_digest;
 
-    if (!EVP_DigestFinal_ex(&mdc_tmp, md_dat, &md_len))
+    if (!EVP_DigestFinal_ex(mdc_tmp, md_dat, &md_len))
       goto err;
     message_digest = PKCS7_digest_from_attributes(sk);
     if (!message_digest)
@@ -792,7 +635,7 @@ int PKCS7_signatureVerify_digest(PKCS7 *p7, PKCS7_SIGNER_INFO *si, X509 *x509,
       ret = -1;
       goto err;
     }
-    if (!EVP_DigestVerifyInit(&mdc_tmp, NULL, EVP_get_digestbynid(md_type), NULL, pkey))
+    if (!EVP_DigestVerifyInit(mdc_tmp, NULL, EVP_get_digestbynid(md_type), NULL, pkey))
       goto err;
 
     alen = ASN1_item_i2d((ASN1_VALUE *) sk, &abuf,
@@ -803,14 +646,14 @@ int PKCS7_signatureVerify_digest(PKCS7 *p7, PKCS7_SIGNER_INFO *si, X509 *x509,
       ret = -1;
       goto err;
     }
-    if (!EVP_VerifyUpdate(&mdc_tmp, abuf, alen))
+    if (!EVP_VerifyUpdate(mdc_tmp, abuf, alen))
       goto err;
 
     OPENSSL_free(abuf);
   }
 
   os = si->enc_digest;
-  i = EVP_VerifyFinal(&mdc_tmp, os->data, os->length, pkey);
+  i = EVP_VerifyFinal(mdc_tmp, os->data, os->length, pkey);
   if (i <= 0)
   {
     PKCS7err(PKCS7_F_PKCS7_SIGNATUREVERIFY, PKCS7_R_SIGNATURE_FAILURE);
@@ -821,118 +664,25 @@ int PKCS7_signatureVerify_digest(PKCS7 *p7, PKCS7_SIGNER_INFO *si, X509 *x509,
     ret = 1;
 err:
   EVP_PKEY_free(pkey);
-  EVP_MD_CTX_cleanup(&mdc);
-  EVP_MD_CTX_cleanup(&mdc_tmp);
+  EVP_MD_CTX_free(mdc);
+  EVP_MD_CTX_free(mdc_tmp);
 
   return (ret);
 }
 
-static LUA_FUNCTION(openssl_pkcs7_verify_digest)
-{
-  PKCS7 *p7 = CHECK_OBJECT(1, PKCS7, "openssl.pkcs7");
-  STACK_OF(X509) *certs = lua_isnoneornil(L, 2) ? NULL : openssl_sk_x509_fromtable(L, 2);
-  X509_STORE *store = lua_isnoneornil(L, 3) ? NULL : CHECK_OBJECT(3, X509_STORE, "openssl.x509_store");
-  size_t len;
-  const char* data = luaL_checklstring(L, 4, &len);
-  long flags = luaL_optint(L, 5, 0);
-  int hash = lua_isnoneornil(L, 6) ? 0 : lua_toboolean(L, 6);
-
-  STACK_OF(X509) *signers;
-  X509 *signer;
-  STACK_OF(PKCS7_SIGNER_INFO) *sinfos;
-  PKCS7_SIGNER_INFO *si;
-  X509_STORE_CTX cert_ctx;
-
-  int i, j = 0, k, ret = 0;
-
-  if (!PKCS7_type_is_signed(p7))
-  {
-    luaL_error(L, "pkcs7 must be signedData");
-  }
-
-  /* Check for no data and no content: no data to verify signature */
-  if (!PKCS7_get_detached(p7))
-  {
-    luaL_error(L, "pkcs7 must be detached signedData");
-  }
-
-
-  sinfos = PKCS7_get_signer_info(p7);
-  if (!sinfos || !sk_PKCS7_SIGNER_INFO_num(sinfos))
-  {
-    luaL_error(L, "pkcs7 signedData without signature");
-  }
-
-  signers = PKCS7_get0_signers(p7, certs, flags);
-  if (!signers)
-  {
-    luaL_error(L, "pkcs7 signedData without signers");
-  }
-
-  if (!store)
-    flags |= PKCS7_NOVERIFY;
-
-  /* Now verify the certificates */
-  if (!(flags & PKCS7_NOVERIFY))
-    for (k = 0; k < sk_X509_num(signers); k++)
-    {
-      signer = sk_X509_value(signers, k);
-      if (!(flags & PKCS7_NOCHAIN))
-      {
-        if (!X509_STORE_CTX_init(&cert_ctx, store, signer,
-                                 p7->d.sign->cert))
-        {
-          PKCS7err(PKCS7_F_PKCS7_VERIFY, ERR_R_X509_LIB);
-          goto err;
-        }
-        X509_STORE_CTX_set_default(&cert_ctx, "smime_sign");
-      }
-      else if (!X509_STORE_CTX_init(&cert_ctx, store, signer, NULL))
-      {
-        PKCS7err(PKCS7_F_PKCS7_VERIFY, ERR_R_X509_LIB);
-        goto err;
-      }
-      if (!(flags & PKCS7_NOCRL))
-        X509_STORE_CTX_set0_crls(&cert_ctx, p7->d.sign->crl);
-      i = X509_verify_cert(&cert_ctx);
-      if (i <= 0)
-        j = X509_STORE_CTX_get_error(&cert_ctx);
-      X509_STORE_CTX_cleanup(&cert_ctx);
-      if (i <= 0)
-      {
-        PKCS7err(PKCS7_F_PKCS7_VERIFY,
-                 PKCS7_R_CERTIFICATE_VERIFY_ERROR);
-        ERR_add_error_data(2, "Verify error:",
-                           X509_verify_cert_error_string(j));
-        goto err;
-      }
-      /* Check for revocation status here */
-    }
-
-  /* Now Verify All Signatures */
-  if (!(flags & PKCS7_NOSIGS))
-    for (i = 0; i < sk_PKCS7_SIGNER_INFO_num(sinfos); i++)
-    {
-      si = sk_PKCS7_SIGNER_INFO_value(sinfos, i);
-      signer = sk_X509_value(signers, i);
-      j = PKCS7_signatureVerify_digest(p7, si, signer,
-                                       (const unsigned char*) data, len, hash);
-      if (j <= 0)
-      {
-        PKCS7err(PKCS7_F_PKCS7_VERIFY, PKCS7_R_SIGNATURE_FAILURE);
-        goto err;
-      }
-    }
-  ret = 1;
-
-err:
-  if (certs)
-    sk_X509_pop_free(certs, X509_free);
-  sk_X509_free(signers);
-  return openssl_pushresult(L, ret);
-}
 #endif
 
+/***
+sign message with signcert and signpkey to create pkcs7 object
+
+@function sign
+@tparam string|bio msg
+@tparam x509 signcert
+@tparam evp_pkey signkey
+@tparam[opt] stack_of_x509 cacerts
+@tparam[opt=0] number flags
+@treturn pkcs7 object
+*/
 static LUA_FUNCTION(openssl_pkcs7_sign)
 {
   BIO *in  = load_bio_object(L, 1);
@@ -961,6 +711,18 @@ static LUA_FUNCTION(openssl_pkcs7_sign)
   return 0;
 }
 
+/***
+verify pkcs7 object, and return msg content, follow by singers
+
+@function verify
+@tparam pkcs7 in
+@tparam[opt] stack_of_x509 signercerts
+@tparam[opt] x509_store cacerts
+@tparam[opt] string|bio msg
+@tparam[opt=0] number flags
+@treturn[1] string content
+@treturn[1] boolean result
+*/
 static LUA_FUNCTION(openssl_pkcs7_verify)
 {
   int ret = 0;
@@ -968,10 +730,14 @@ static LUA_FUNCTION(openssl_pkcs7_verify)
   STACK_OF(X509) *signers = lua_isnoneornil(L, 2) ? NULL : openssl_sk_x509_fromtable(L, 2);
   X509_STORE *store = lua_isnoneornil(L, 3) ? NULL : CHECK_OBJECT(3, X509_STORE, "openssl.x509_store");
   BIO* in = lua_isnoneornil(L, 4) ? NULL : load_bio_object(L, 4);
-  long flags = luaL_optint(L, 5, 0);
-  BIO* out = BIO_new(BIO_s_mem());
+  long flags = luaL_optint(L, 5, PKCS7_DETACHED);
+  BIO* out = NULL;
+
   if (!store)
     flags |= PKCS7_NOVERIFY;
+  if ((flags&PKCS7_DETACHED) != 0)
+    out = BIO_new(BIO_s_mem());
+
   if (PKCS7_verify(p7, signers, store, in, out, flags) == 1)
   {
     if (out && (flags & PKCS7_DETACHED) == 0)
@@ -1000,6 +766,15 @@ static LUA_FUNCTION(openssl_pkcs7_verify)
   return ret;
 }
 
+/***
+encrypt message with recipcerts certificates return encrypted pkcs7 object
+
+@function encrypt
+@tparam string|bio msg
+@tparam stack_of_x509 recipcerts
+@tparam[opt='rc4'] string|evp_cipher cipher
+@tparam[opt] number flags
+*/
 static LUA_FUNCTION(openssl_pkcs7_encrypt)
 {
   PKCS7 * p7 = NULL;
@@ -1028,6 +803,15 @@ static LUA_FUNCTION(openssl_pkcs7_encrypt)
   return 1;
 }
 
+/***
+decrypt encrypted pkcs7 message
+
+@function decrypt
+@tparam pkcs7 input
+@tparam x509 recipcert
+@tparam evp_pkey recipkey
+@treturn string decrypt message
+*/
 static LUA_FUNCTION(openssl_pkcs7_decrypt)
 {
   PKCS7 *p7 = CHECK_OBJECT(1, PKCS7, "openssl.pkcs7");
@@ -1048,7 +832,11 @@ static LUA_FUNCTION(openssl_pkcs7_decrypt)
   return 1;
 }
 
-/*** pkcs7 object method ***/
+/***
+openssl.pkcs7 object
+
+@type pkcs7
+*/
 static LUA_FUNCTION(openssl_pkcs7_gc)
 {
   PKCS7* p7 = CHECK_OBJECT(1, PKCS7, "openssl.pkcs7");
@@ -1056,17 +844,26 @@ static LUA_FUNCTION(openssl_pkcs7_gc)
   return 0;
 }
 
+/***
+export pkcs7 as string
+
+@function export
+@tparam[opt='pem'] string support export as 'pem' or 'der' format, default is 'pem'
+@treturn string
+*/
 static LUA_FUNCTION(openssl_pkcs7_export)
 {
-  int pem;
   PKCS7 * p7 = CHECK_OBJECT(1, PKCS7, "openssl.pkcs7");
-  int top = lua_gettop(L);
   BIO* bio_out = NULL;
-
-  pem = top > 1 ? lua_toboolean(L, 2) : 1;
+  int fmt = lua_type(L, 2);
+  luaL_argcheck(L, fmt == LUA_TSTRING || fmt == LUA_TNONE, 2,
+                "only accept 'pem','der' or none");
+  fmt = luaL_checkoption(L, 2, "pem", format);
+  luaL_argcheck(L, fmt == FORMAT_PEM || fmt == FORMAT_DER, 2,
+                "only accept pem or der, default is pem");
 
   bio_out  = BIO_new(BIO_s_mem());
-  if (pem)
+  if (fmt == FORMAT_PEM)
   {
 
     if (PEM_write_bio_PKCS7(bio_out, p7))
@@ -1128,19 +925,28 @@ static int openssl_push_pkcs7_signer_info(lua_State *L, PKCS7_SIGNER_INFO *info)
 
   if (info->pkey)
   {
-    CRYPTO_add(&info->pkey->references, 1, CRYPTO_LOCK_EVP_PKEY);
+    EVP_PKEY_up_ref(info->pkey);
     AUXILIAR_SETOBJECT(L, info->pkey, "openssl.evp_pkey", -1, "pkey");
   }
   return 1;
 }
 
-static LUA_FUNCTION(openssl_pkcs7_signer_info_gc)
+static LUA_FUNCTION(openssl_pkcs7_type)
 {
-  PKCS7_SIGNER_INFO *info = CHECK_OBJECT(1, PKCS7_SIGNER_INFO, "openssl.pkcs7_signer_info");
-  PKCS7_SIGNER_INFO_free(info);
-  return 0;
+  PKCS7 * p7 = CHECK_OBJECT(1, PKCS7, "openssl.pkcs7");
+  int i = OBJ_obj2nid(p7->type);
+
+  lua_pushstring(L, OBJ_nid2sn(i));
+  lua_pushstring(L, OBJ_nid2ln(i));
+  return 2;
 }
 
+/***
+export pkcs7 as a string
+
+@function parse
+@treturn table  a table has pkcs7 infomation, include type,and other things relate to types
+*/
 static LUA_FUNCTION(openssl_pkcs7_parse)
 {
   PKCS7 * p7 = CHECK_OBJECT(1, PKCS7, "openssl.pkcs7");
@@ -1250,8 +1056,412 @@ static LUA_FUNCTION(openssl_pkcs7_parse)
   return 1;
 }
 
+/***
+pkcs7 sign add signer
+
+@function add_signer
+@tparam x509 cert used to sign data
+@tparam evp_pkey pkey used to sign data
+@tparam evp_md|int digest method when sign data
+@tparam[opt=0] int flags switch process when add signer
+@treturn boolean result true for success
+*/
+static LUA_FUNCTION(openssl_pkcs7_sign_add_signer)
+{
+  PKCS7 *p7 = CHECK_OBJECT(1, PKCS7, "openssl.pkcs7");
+  X509 *signcert = CHECK_OBJECT(2, X509, "openssl.x509");
+  EVP_PKEY *pkey = CHECK_OBJECT(3, EVP_PKEY, "openssl.evp_pkey");
+  const EVP_MD* md = get_digest(L, 4, NULL);
+  long flags = luaL_optint(L, 5, 0);
+  PKCS7_SIGNER_INFO *signer = 0;
+
+  luaL_argcheck(L, X509_check_private_key(signcert, pkey), 3,
+                "sigcert and private key not match");
+
+  signer = PKCS7_sign_add_signer(p7, signcert, pkey, md, flags);
+  (void) signer;
+  return openssl_pushresult(L, signcert != NULL ? 1 : 0);
+}
+
+#if OPENSSL_VERSION_NUMBER > 0x10000000L
+/***
+pkcs7 sign hash data
+@function sign_digest
+@tparam string data to sign data, maybe already hashed
+@tparam[opt=0] int flags when sign data
+@tparam[opt=false] boolean hashed when true will skip hash process
+@treturn boolean result true for success
+*/
+static LUA_FUNCTION(openssl_pkcs7_sign_digest)
+{
+  PKCS7 *p7 = CHECK_OBJECT(1, PKCS7, "openssl.pkcs7");
+  size_t l;
+  const char* data = luaL_checklstring(L, 2, &l);
+  long flags = luaL_optint(L, 3, 0);
+  int hash = lua_isnoneornil(L, 4) ? 0 : lua_toboolean(L, 4);
+
+  int ret = 0;
+  int i, j;
+
+  const EVP_MD* md;
+  PKCS7_SIGNER_INFO *si;
+  EVP_MD_CTX *mdc;
+  STACK_OF(X509_ATTRIBUTE) *sk;
+  STACK_OF(PKCS7_SIGNER_INFO) *si_sk = NULL;
+  ASN1_OCTET_STRING *os = NULL;
+
+  if (p7->d.ptr == NULL)
+  {
+    luaL_error(L, "pkcs7 without content");
+    return 0;
+  }
+
+  if (flags & PKCS7_DETACHED)
+  {
+    PKCS7_set_detached(p7, 1);
+  }
+
+  mdc = EVP_MD_CTX_new();
+  EVP_MD_CTX_init(mdc);
+  i = OBJ_obj2nid(p7->type);
+  p7->state = PKCS7_S_HEADER;
+
+  switch (i)
+  {
+  case NID_pkcs7_data:
+    os = p7->d.data;
+    break;
+  case NID_pkcs7_signedAndEnveloped:
+    /* XXXXXXXXXXXXXXXX */
+    si_sk = p7->d.signed_and_enveloped->signer_info;
+    os = p7->d.signed_and_enveloped->enc_data->enc_data;
+    if (!os)
+    {
+#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
+      os = M_ASN1_OCTET_STRING_new();
+#else
+      os = ASN1_OCTET_STRING_new();
+#endif
+      if (!os)
+      {
+        PKCS7err(PKCS7_F_PKCS7_DATAFINAL, ERR_R_MALLOC_FAILURE);
+        goto err;
+      }
+      p7->d.signed_and_enveloped->enc_data->enc_data = os;
+    }
+    break;
+  case NID_pkcs7_enveloped:
+    /* XXXXXXXXXXXXXXXX */
+    os = p7->d.enveloped->enc_data->enc_data;
+    if (!os)
+    {
+#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
+      os = M_ASN1_OCTET_STRING_new();
+#else
+      os = ASN1_OCTET_STRING_new();
+#endif
+      if (!os)
+      {
+        PKCS7err(PKCS7_F_PKCS7_DATAFINAL, ERR_R_MALLOC_FAILURE);
+        goto err;
+      }
+      p7->d.enveloped->enc_data->enc_data = os;
+    }
+    break;
+  case NID_pkcs7_signed:
+    si_sk = p7->d.sign->signer_info;
+    os = PKCS7_get_octet_string(p7->d.sign->contents);
+    /* If detached data then the content is excluded */
+    if (PKCS7_type_is_data(p7->d.sign->contents) && p7->detached)
+    {
+#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
+      M_ASN1_OCTET_STRING_free(os);
+#else
+      ASN1_OCTET_STRING_free(os);
+#endif
+      os = NULL;
+      p7->d.sign->contents->d.data = NULL;
+    }
+    break;
+
+  case NID_pkcs7_digest:
+    os = PKCS7_get_octet_string(p7->d.digest->contents);
+    /* If detached data then the content is excluded */
+    if (PKCS7_type_is_data(p7->d.digest->contents) && p7->detached)
+    {
+#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
+      M_ASN1_OCTET_STRING_free(os);
+#else
+      ASN1_OCTET_STRING_free(os);
+#endif
+      os = NULL;
+      p7->d.digest->contents->d.data = NULL;
+    }
+    break;
+
+  default:
+    PKCS7err(PKCS7_F_PKCS7_DATAFINAL, PKCS7_R_UNSUPPORTED_CONTENT_TYPE);
+    goto err;
+  }
+
+  if (si_sk != NULL)
+  {
+    for (i = 0; i < sk_PKCS7_SIGNER_INFO_num(si_sk); i++)
+    {
+      si = sk_PKCS7_SIGNER_INFO_value(si_sk, i);
+      if (si->pkey == NULL)
+        continue;
+      j = OBJ_obj2nid(si->digest_alg->algorithm);
+      md = EVP_get_digestbynid(j);
+      EVP_DigestInit_ex(mdc, md, NULL);
+
+      if (hash)
+      {
+#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
+        if (l == (size_t) mdc->digest->ctx_size)
+        {
+          memcpy(mdc->md_data, data, l);
+        }
+#else
+        if (l == (size_t)EVP_MD_meth_get_app_datasize(EVP_MD_CTX_md(mdc)))
+        {
+          memcpy(EVP_MD_CTX_md_data(mdc), data, l);
+        }
+#endif
+        else
+        {
+          EVP_MD_CTX_free(mdc);
+          luaL_argerror(L, 2, "data with wrong length");
+        }
+      }
+      else
+        EVP_DigestUpdate(mdc, data, l);
+
+      sk = si->auth_attr;
+
+      /*
+      * If there are attributes, we add the digest attribute and only
+      * sign the attributes
+      */
+      if (sk_X509_ATTRIBUTE_num(sk) > 0)
+      {
+        if (!do_pkcs7_signed_attrib(si, mdc))
+          goto err;
+      }
+      else
+      {
+        unsigned char *abuf = NULL;
+        unsigned int abuflen;
+        abuflen = EVP_PKEY_size(si->pkey);
+        abuf = OPENSSL_malloc(abuflen);
+        if (!abuf)
+          goto err;
+
+        if (!EVP_SignFinal(mdc, abuf, &abuflen, si->pkey))
+        {
+          PKCS7err(PKCS7_F_PKCS7_DATAFINAL, ERR_R_EVP_LIB);
+          goto err;
+        }
+        ASN1_STRING_set0(si->enc_digest, abuf, abuflen);
+      }
+    }
+  }
+  else if (i == NID_pkcs7_digest)
+  {
+    unsigned char md_data[EVP_MAX_MD_SIZE];
+    unsigned int md_len;
+    md = EVP_get_digestbynid(OBJ_obj2nid(p7->d.digest->md->algorithm));
+    EVP_DigestInit_ex(mdc, md, NULL);
+#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
+    if (l == (size_t) mdc->digest->ctx_size)
+    {
+      memcpy(mdc->md_data, data, l);
+    }
+#else
+    if (l == (size_t)EVP_MD_meth_get_app_datasize(EVP_MD_CTX_md(mdc)))
+    {
+      memcpy(EVP_MD_CTX_md_data(mdc), data, l);
+    }
+#endif
+    else
+    {
+      EVP_MD_CTX_free(mdc);
+      luaL_error(L, "data with wrong data");
+    }
+    if (!EVP_DigestFinal_ex(mdc, md_data, &md_len))
+      goto err;
+    ASN1_OCTET_STRING_set(p7->d.digest->digest, md_data, md_len);
+  }
+
+  if (!PKCS7_is_detached(p7))
+  {
+    /*
+    * NOTE(emilia): I think we only reach os == NULL here because detached
+    * digested data support is broken.
+    */
+    if (os == NULL)
+      goto err;
+    if (!(os->flags & ASN1_STRING_FLAG_NDEF))
+    {
+      char *cont = memdup(data, l);
+      long contlen = l;
+      ASN1_STRING_set0(os, (unsigned char *) cont, contlen);
+    }
+  }
+
+  ret = 1;
+err:
+  EVP_MD_CTX_free(mdc);
+  return openssl_pushresult(L, ret);
+}
+
+/***
+pkcs7 verify signature or digest
+
+@function verify_digest
+@tparam[opt] table certs contains certificate used to sign data
+@tparam[opt] x509_store store to verify certs
+@tparam string data to be signed
+@tparam[opt=false] boolean hashed true for data already hashed
+@treturn boolean result true for success
+*/
+static LUA_FUNCTION(openssl_pkcs7_verify_digest)
+{
+  PKCS7 *p7 = CHECK_OBJECT(1, PKCS7, "openssl.pkcs7");
+  STACK_OF(X509) *certs = lua_isnoneornil(L, 2) ? NULL : openssl_sk_x509_fromtable(L, 2);
+  X509_STORE *store = lua_isnoneornil(L, 3) ? NULL : CHECK_OBJECT(3, X509_STORE, "openssl.x509_store");
+  size_t len = 0;
+  const char* data = luaL_optlstring(L, 4, NULL, &len);
+  long flags = luaL_optint(L, 5, 0);
+  int hash = lua_isnoneornil(L, 6) ? 0 : lua_toboolean(L, 6);
+
+  STACK_OF(X509) *signers;
+  X509 *signer;
+  STACK_OF(PKCS7_SIGNER_INFO) *sinfos;
+  PKCS7_SIGNER_INFO *si;
+  X509_STORE_CTX *cert_ctx;
+
+  int i, j = 0, k, ret = 0;
+
+  /* Check for no data and no content: no data to verify signature */
+  if (flags & PKCS7_DETACHED)
+  {
+    if (!PKCS7_get_detached(p7))
+    {
+      luaL_error(L, "pkcs7 must be detached signedData");
+    }
+    luaL_argcheck(L, data != NULL, 4, "need data to be verified");
+  }
+  else
+  {
+    ASN1_OCTET_STRING *os = NULL;
+
+    luaL_argcheck(L, data == NULL, 4, "must be nil or none");
+    os = PKCS7_get_octet_string(p7->d.sign->contents);
+    data = (const char*)os->data;
+    len = os->length;
+  }
+
+  sinfos = PKCS7_get_signer_info(p7);
+  if (!sinfos || !sk_PKCS7_SIGNER_INFO_num(sinfos))
+  {
+    luaL_error(L, "pkcs7 signedData without signature");
+  }
+
+  signers = PKCS7_get0_signers(p7, certs, flags);
+  if (!signers)
+  {
+    luaL_error(L, "pkcs7 signedData without signers");
+  }
+
+  if (!store)
+    flags |= PKCS7_NOVERIFY;
+
+  /* Now verify the certificates */
+  if (!(flags & PKCS7_NOVERIFY))
+    for (k = 0; k < sk_X509_num(signers); k++)
+    {
+      signer = sk_X509_value(signers, k);
+      cert_ctx = X509_STORE_CTX_new();
+      if (!(flags & PKCS7_NOCHAIN))
+      {
+        if (!X509_STORE_CTX_init(cert_ctx, store, signer,
+                                 p7->d.sign->cert))
+        {
+          PKCS7err(PKCS7_F_PKCS7_VERIFY, ERR_R_X509_LIB);
+          goto err;
+        }
+        X509_STORE_CTX_set_default(cert_ctx, "smime_sign");
+      }
+      else if (!X509_STORE_CTX_init(cert_ctx, store, signer, NULL))
+      {
+        PKCS7err(PKCS7_F_PKCS7_VERIFY, ERR_R_X509_LIB);
+        goto err;
+      }
+      if (!(flags & PKCS7_NOCRL))
+        X509_STORE_CTX_set0_crls(cert_ctx, p7->d.sign->crl);
+      i = X509_verify_cert(cert_ctx);
+      if (i <= 0)
+        j = X509_STORE_CTX_get_error(cert_ctx);
+      X509_STORE_CTX_free(cert_ctx);
+      if (i <= 0)
+      {
+        PKCS7err(PKCS7_F_PKCS7_VERIFY,
+                 PKCS7_R_CERTIFICATE_VERIFY_ERROR);
+        ERR_add_error_data(2, "Verify error:",
+                           X509_verify_cert_error_string(j));
+        goto err;
+      }
+      /* Check for revocation status here */
+    }
+
+  /* Now Verify All Signatures */
+  if (!(flags & PKCS7_NOSIGS))
+    for (i = 0; i < sk_PKCS7_SIGNER_INFO_num(sinfos); i++)
+    {
+      si = sk_PKCS7_SIGNER_INFO_value(sinfos, i);
+      signer = sk_X509_value(signers, i);
+      j = PKCS7_signatureVerify_digest(p7, si, signer,
+                                       (const unsigned char*) data, len, hash);
+      if (j <= 0)
+      {
+        PKCS7err(PKCS7_F_PKCS7_VERIFY, PKCS7_R_SIGNATURE_FAILURE);
+        goto err;
+      }
+    }
+  ret = 1;
+
+err:
+  if (certs)
+    sk_X509_pop_free(certs, X509_free);
+  sk_X509_free(signers);
+  return openssl_pushresult(L, ret);
+}
+#endif
+/***
+verify pkcs7 object, and return msg content, follow by singers
+
+@function verify
+@tparam[opt] stack_of_x509 signercerts
+@tparam[opt] x509_store cacerts
+@tparam[opt] string|bio msg
+@tparam[opt=0] number flags
+@treturn string content
+@treturn stack_of_x509 signers
+*/
+
+/***
+decrypt encrypted pkcs7 message
+
+@function decrypt
+@tparam x509 recipcert
+@tparam evp_pkey recipkey
+@treturn string decrypt message
+*/
+
 static luaL_Reg pkcs7_funcs[] =
 {
+  {"type",          openssl_pkcs7_type},
   {"parse",         openssl_pkcs7_parse},
   {"export",        openssl_pkcs7_export},
   {"decrypt",       openssl_pkcs7_decrypt},
@@ -1284,7 +1494,7 @@ static const luaL_Reg R[] =
   {NULL,  NULL}
 };
 
-static LuaL_Enum pkcs7_const[] =
+static LuaL_Enumeration pkcs7_const[] =
 {
   {"TEXT",         PKCS7_TEXT},
   {"NOCERTS",      PKCS7_NOCERTS},
@@ -1308,7 +1518,6 @@ static LuaL_Enum pkcs7_const[] =
 
 int luaopen_pkcs7(lua_State *L)
 {
-  int i;
   auxiliar_newclass(L, "openssl.pkcs7", pkcs7_funcs);
 
   lua_newtable(L);
@@ -1318,11 +1527,6 @@ int luaopen_pkcs7(lua_State *L)
   lua_pushliteral(L, MYVERSION);
   lua_settable(L, -3);
 
-  for (i = 0; i < sizeof(pkcs7_const) / sizeof(LuaL_Enum) - 1; i++)
-  {
-    LuaL_Enum e = pkcs7_const[i];
-    lua_pushinteger(L, e.val);
-    lua_setfield(L, -2, e.name);
-  }
+  auxiliar_enumerate(L, -1, pkcs7_const);
   return 1;
 }
