@@ -1,9 +1,12 @@
-/*=========================================================================*\
-* x509.c
-* x509 modules for lua-openssl binding
-*
-* Author:  george zhao <zhaozg(at)gmail.com>
-\*=========================================================================*/
+/***
+x509 modules for lua-openssl binding
+create and manage x509 certificate
+@module x509
+@usage
+ x509 = require'openssl'.x509
+*/
+
+
 #include "openssl.h"
 #include "private.h"
 #define CRYPTO_LOCK_REF
@@ -13,21 +16,326 @@
 #define MYVERSION MYNAME " library for " LUA_VERSION " / Nov 2014 / "\
   "based on OpenSSL " SHLIB_VERSION_NUMBER
 
-int openssl_push_x509_algor(lua_State*L, const X509_ALGOR* alg)
+static int openssl_push_purpose(lua_State*L, X509_PURPOSE* purpose)
 {
   lua_newtable(L);
-  openssl_push_asn1object(L, alg->algorithm);
-  lua_setfield(L, -2, "algorithm");
-  if (alg->parameter)
-  {
-    openssl_push_asn1type(L, alg->parameter);
-    lua_setfield(L, -2, "parameter");
-  }
+
+  AUXILIAR_SET(L, -1, "purpose", purpose->purpose, integer);
+  AUXILIAR_SET(L, -1, "trust", purpose->trust, integer);
+  AUXILIAR_SET(L, -1, "flags", purpose->flags, integer);
+
+  AUXILIAR_SET(L, -1, "name", purpose->name, string);
+  AUXILIAR_SET(L, -1, "sname", purpose->sname, string);
+
   return 1;
 };
 
+/***
+return all supported purpose as table
+@function purpose
+@treturn table
+*/
+/*
+get special purpose info as table
+@function purpose
+@tparam number|string purpose id or short name
+@treturn table
+*/
+static int openssl_x509_purpose(lua_State*L)
+{
+  if (lua_isnoneornil(L, 1))
+  {
+    int count = X509_PURPOSE_get_count();
+    int i;
+    lua_newtable(L);
+    for (i = 0; i < count; i++)
+    {
+      X509_PURPOSE* purpose = X509_PURPOSE_get0(i);
+      openssl_push_purpose(L, purpose);
+      lua_rawseti(L, -2, i + 1);
+    }
+    return 1;
+  }
+  else if (lua_isnumber(L, 1))
+  {
+    int idx = X509_PURPOSE_get_by_id(lua_tointeger(L, 1));
+    if (idx >= 0)
+    {
+      X509_PURPOSE* purpose = X509_PURPOSE_get0(idx);
+      openssl_push_purpose(L, purpose);
+    }
+    else
+      lua_pushnil(L);
+    return 1;
+  }
+  else if (lua_isstring(L, 1))
+  {
+    char* name = (char*)lua_tostring(L, 1);
+    int idx = X509_PURPOSE_get_by_sname(name);
+    if (idx >= 0)
+    {
+      X509_PURPOSE* purpose = X509_PURPOSE_get0(idx);
+      openssl_push_purpose(L, purpose);
+    }
+    else
+      lua_pushnil(L);
+    return 1;
+  }
+  return 0;
+};
+
+static const char* usage_mode[] =
+{
+  "standard",
+  "netscape",
+  "extend",
+  NULL
+};
+
+/***
+get support certtypes
+@function certtypes
+@tparam[opt='standard'] string type support 'standard','netscape','extend'
+@treturn table if type is 'standard' or 'netscape', contains node with {lname=...,sname=...,bitname=...},
+               if type is 'extend', contains node with {lname=...,sname=...,nid=...}
+*/
+static int openssl_x509_certtypes(lua_State*L)
+{
+  int mode = luaL_checkoption(L, 1, "standard", usage_mode);
+  int i;
+  const BIT_STRING_BITNAME* bitname;
+
+  switch (mode)
+  {
+  case 0:
+  {
+    const static BIT_STRING_BITNAME key_usage_type_table[] =
+    {
+      {0, "Digital Signature", "digitalSignature"},
+      {1, "Non Repudiation", "nonRepudiation"},
+      {2, "Key Encipherment", "keyEncipherment"},
+      {3, "Data Encipherment", "dataEncipherment"},
+      {4, "Key Agreement", "keyAgreement"},
+      {5, "Certificate Sign", "keyCertSign"},
+      {6, "CRL Sign", "cRLSign"},
+      {7, "Encipher Only", "encipherOnly"},
+      {8, "Decipher Only", "decipherOnly"},
+      { -1, NULL, NULL}
+    };
+    lua_newtable(L);
+    for (i = 0, bitname = &key_usage_type_table[i]; bitname->bitnum != -1; i++, bitname = &key_usage_type_table[i])
+    {
+      openssl_push_bit_string_bitname(L, bitname);
+      lua_rawseti(L, -2, i + 1);
+    }
+    return 1;
+
+  }
+  case 1:
+  {
+    const static BIT_STRING_BITNAME ns_cert_type_table[] =
+    {
+      {0, "SSL Client", "client"},
+      {1, "SSL Server", "server"},
+      {2, "S/MIME", "email"},
+      {3, "Object Signing", "objsign"},
+      {4, "Unused", "reserved"},
+      {5, "SSL CA", "sslCA"},
+      {6, "S/MIME CA", "emailCA"},
+      {7, "Object Signing CA", "objCA"},
+      { -1, NULL, NULL}
+    };
+    lua_newtable(L);
+    for (i = 0, bitname = &ns_cert_type_table[i]; bitname->bitnum != -1; i++, bitname = &ns_cert_type_table[i])
+    {
+      openssl_push_bit_string_bitname(L, bitname);
+      lua_rawseti(L, -2, i + 1);
+    }
+    return 1;
+  }
+  case 2:
+  {
+    static const int ext_nids[] =
+    {
+      NID_server_auth,
+      NID_client_auth,
+      NID_email_protect,
+      NID_code_sign,
+      NID_ms_sgc,
+      NID_ns_sgc,
+      NID_OCSP_sign,
+      NID_time_stamp,
+      NID_dvcs,
+      NID_anyExtendedKeyUsage
+    };
+    int count = sizeof(ext_nids) / sizeof(int);
+    int nid;
+    lua_newtable(L);
+    for (i = 0; i < count; i++)
+    {
+      nid = ext_nids[i];
+      lua_newtable(L);
+      lua_pushstring(L, OBJ_nid2ln(nid));
+      lua_setfield(L, -2, "lname");
+      lua_pushstring(L, OBJ_nid2sn(nid));
+      lua_setfield(L, -2, "sname");
+      lua_pushinteger(L, nid);
+      lua_setfield(L, -2, "nid");
+      lua_rawseti(L, -2, i + 1);
+    };
+    return 1;
+  }
+  }
+  return 0;
+}
+
+/***
+get certificate verify result string message
+@function verify_cert_error_string
+@tparam number verify_result
+@treturn string result message
+*/
+static int openssl_verify_cert_error_string(lua_State*L)
+{
+  int v = luaL_checkint(L, 1);
+  const char*s = X509_verify_cert_error_string(v);
+  lua_pushstring(L, s);
+  return 1;
+}
+
+/***
+read x509 from string or bio input
+@function read
+@tparam bio|string input input data
+@tparam[opt='auto'] string format support 'auto','pem','der'
+@treturn x509 certificate object
+*/
+static LUA_FUNCTION(openssl_x509_read)
+{
+  X509 *cert = NULL;
+  BIO *in = load_bio_object(L, 1);
+  int fmt = luaL_checkoption(L, 2, "auto", format);
+  if (fmt == FORMAT_AUTO)
+  {
+    fmt = bio_is_der(in) ? FORMAT_DER : FORMAT_PEM;
+  }
+
+  if (fmt == FORMAT_DER)
+  {
+    cert = d2i_X509_bio(in, NULL);
+  }
+  else if (fmt == FORMAT_PEM)
+  {
+    cert = PEM_read_bio_X509(in, NULL, NULL, NULL);
+  }
+
+  BIO_free(in);
+
+  if (cert)
+  {
+    PUSH_OBJECT(cert, "openssl.x509");
+    return 1;
+  }
+  return openssl_pushresult(L, 0);
+}
+
+/***
+create or generate a new x509 object.
+@function new
+@tparam[opt] openssl.bn serial serial number
+@tparam[opt] x509_req csr,copy x509_name, pubkey and extension to new object
+@tparam[opt] x509_name subject subject name set to x509_req
+@tparam[opt] stack_of_x509_extension extensions add to x509
+@tparam[opt] stack_of_x509_attribute attributes add to x509
+@treturn x509 certificate object
+*/
+static int openssl_x509_new(lua_State* L)
+{
+  int i = 1;
+  int ret = 1;
+  int n = lua_gettop(L);
+  X509 *x = X509_new();
+
+  ret = X509_set_version(x, 2);
+  if (ret == 1 && (
+        auxiliar_getclassudata(L, "openssl.bn", i) ||
+        lua_isstring(L, i) || lua_isnumber(L, i)
+      ))
+  {
+    BIGNUM *bn = BN_get(L, i);
+    ASN1_INTEGER* ai = BN_to_ASN1_INTEGER(bn, NULL);
+    BN_free(bn);
+    ret = X509_set_serialNumber(x, ai);
+    ASN1_INTEGER_free(ai);
+    i++;
+  }
+
+  for (; i <= n; i++)
+  {
+    if (ret == 1 && auxiliar_getclassudata(L, "openssl.x509_req", i))
+    {
+      X509_REQ* csr = CHECK_OBJECT(i, X509_REQ, "openssl.x509_req");
+      X509_NAME* xn = X509_REQ_get_subject_name(csr);
+      ret = X509_set_subject_name(x, xn);
+
+      if (ret == 1)
+      {
+        STACK_OF(X509_EXTENSION) *exts = X509_REQ_get_extensions(csr);
+        int j, n1;
+        n1 = sk_X509_EXTENSION_num(exts);
+        for (j = 0; ret == 1 && j < n1; j++)
+        {
+          ret = X509_add_ext(x, sk_X509_EXTENSION_value(exts, j), j);
+        }
+        sk_X509_EXTENSION_pop_free(exts, X509_EXTENSION_free);
+      }
+      if (ret == 1)
+      {
+        EVP_PKEY* pkey = X509_REQ_get_pubkey(csr);
+        ret = X509_set_pubkey(x, pkey);
+        EVP_PKEY_free(pkey);
+      }
+      i++;
+    };
+
+    if (ret == 1 && auxiliar_getclassudata(L, "openssl.x509_name", i))
+    {
+      X509_NAME *xn = CHECK_OBJECT(i, X509_NAME, "openssl.x509_name");
+      ret = X509_set_subject_name(x, xn);
+      i++;
+    }
+  }
+
+  if (ret == 1)
+  {
+    PUSH_OBJECT(x, "openssl.x509");
+    return 1;
+  }
+  else
+  {
+    X509_free(x);
+    return openssl_pushresult(L, ret);
+  }
+};
+
+static luaL_Reg R[] =
+{
+  {"new",           openssl_x509_new },
+  {"read",          openssl_x509_read },
+  {"purpose",       openssl_x509_purpose },
+  {"certtypes",     openssl_x509_certtypes },
+  {"verify_cert_error_string", openssl_verify_cert_error_string },
+
+  {NULL,    NULL}
+};
+
 int openssl_push_general_name(lua_State*L, const GENERAL_NAME* general_name)
 {
+  if (general_name == NULL)
+  {
+    lua_pushnil(L);
+    return 1;
+  }
   lua_newtable(L);
 
   switch (general_name->type)
@@ -137,35 +445,17 @@ static int check_cert(X509_STORE *ca, X509 *x, STACK_OF(X509) *untrustedchain, i
   return X509_V_ERR_OUT_OF_MEM;
 }
 
-static LUA_FUNCTION(openssl_x509_read)
-{
-  X509 *cert = NULL;
-  BIO *in = load_bio_object(L, 1);
-  int fmt = luaL_checkoption(L, 2, "auto", format);
-  if (fmt == FORMAT_AUTO)
-  {
-    fmt = bio_is_der(in) ? FORMAT_DER : FORMAT_PEM;
-  }
-
-  if (fmt == FORMAT_DER)
-  {
-    cert = d2i_X509_bio(in, NULL);
-  }
-  else if (fmt == FORMAT_PEM)
-  {
-    cert = PEM_read_bio_X509(in, NULL, NULL, NULL);
-  }
-
-  BIO_free(in);
-
-  if (cert)
-  {
-    PUSH_OBJECT(cert, "openssl.x509");
-    return 1;
-  }
-  return openssl_pushresult(L, 0);
-}
-
+/***
+openssl.x509 object
+@type x509
+*/
+/***
+export x509_req to string
+@function export
+@tparam[opt='pem'] string format, 'der' or 'pem' default
+@tparam[opt='true'] boolean noext not export extension
+@treturn string
+*/
 static LUA_FUNCTION(openssl_x509_export)
 {
   X509 *cert = CHECK_OBJECT(1, X509, "openssl.x509");
@@ -211,19 +501,25 @@ static LUA_FUNCTION(openssl_x509_export)
   return 1;
 };
 
-
+/***
+parse x509 object as table
+@function parse
+@tparam[opt=true] shortname default will use short object name
+@treturn table result which all x509 information
+*/
 static LUA_FUNCTION(openssl_x509_parse)
 {
   int i;
   X509 * cert = CHECK_OBJECT(1, X509, "openssl.x509");
   X509_ALGOR* alg = 0;
   lua_newtable(L);
-
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
   if (cert->name)
   {
     AUXILIAR_SET(L, -1, "name", cert->name, string);
   }
   AUXILIAR_SET(L, -1, "valid", cert->valid, boolean);
+#endif
   AUXILIAR_SET(L, -1, "version", X509_get_version(cert), integer);
 
   openssl_push_xname_asobject(L, X509_get_subject_name(cert));
@@ -236,16 +532,31 @@ static LUA_FUNCTION(openssl_x509_parse)
     AUXILIAR_SET(L, -1, "hash", buf, string);
   }
 
-  PUSH_ASN1_INTEGER(L, cert->cert_info->serialNumber);
+  PUSH_ASN1_INTEGER(L, X509_get0_serialNumber(cert));
   lua_setfield(L, -2, "serialNumber");
+
   PUSH_ASN1_TIME(L, X509_get_notBefore(cert));
   lua_setfield(L, -2, "notBefore");
   PUSH_ASN1_TIME(L, X509_get_notAfter(cert));
   lua_setfield(L, -2, "notAfter");
 
-  alg = X509_ALGOR_dup(cert->sig_alg);
-  PUSH_OBJECT(alg, "openssl.x509_algor");
-  lua_setfield(L, -2, "sig_alg");
+  {
+    CONSTIFY_X509_get0 X509_ALGOR *palg = NULL;
+    CONSTIFY_X509_get0 ASN1_BIT_STRING *psig = NULL;
+
+    X509_get0_signature(&psig, &palg, cert);
+    if (palg != NULL)
+    {
+      alg = X509_ALGOR_dup((X509_ALGOR*)palg);
+      PUSH_OBJECT(alg, "openssl.x509_algor");
+      lua_setfield(L, -2, "sig_alg");
+    }
+    if (psig != NULL)
+    {
+      lua_pushlstring(L, (const char *)psig->data, psig->length);
+      lua_setfield(L, -2, "sig");
+    }
+  }
 
   {
     int l = 0;
@@ -310,6 +621,17 @@ static LUA_FUNCTION(openssl_x509_free)
   return 0;
 }
 
+/***
+get public key of x509
+@function pubkey
+@treturn evp_pkey public key
+*/
+/***
+set public key of x509
+@function pubkey
+@tparam evp_pkey pubkey public key set to x509
+@treturn boolean result, true for success
+*/
 static LUA_FUNCTION(openssl_x509_public_key)
 {
   X509 *cert = CHECK_OBJECT(1, X509, "openssl.x509");
@@ -327,6 +649,7 @@ static LUA_FUNCTION(openssl_x509_public_key)
   }
 }
 
+#if 0
 static int verify_cb(int ok, X509_STORE_CTX *ctx)
 {
   int err;
@@ -358,11 +681,29 @@ static int verify_cb(int ok, X509_STORE_CTX *ctx)
     return 1;
   }
 }
+#endif
 
+/***
+check x509 with ca certchian and option purpose
+purpose can be one of: ssl_client, ssl_server, ns_ssl_server, smime_sign, smime_encrypt, crl_sign, any, ocsp_helper, timestamp_sign
+@function check
+@tparam x509_store cacerts
+@tparam x509_store untrusted certs  containing a bunch of certs that are not trusted but may be useful in validating the certificate.
+@tparam[opt] string purpose to check supported
+@treturn boolean result true for check pass
+@treturn integer verify result
+@see verify_cert_error_string
+*/
+/***
+check x509 with evp_pkey
+@function check
+@tparam evp_pkey pkey private key witch match with x509 pubkey
+@treturn boolean result true for check pass
+*/
 static LUA_FUNCTION(openssl_x509_check)
 {
   X509 * cert = CHECK_OBJECT(1, X509, "openssl.x509");
-  if (auxiliar_isclass(L, "openssl.evp_pkey", 2))
+  if (auxiliar_getclassudata(L, "openssl.evp_pkey", 2))
   {
     EVP_PKEY * key = CHECK_OBJECT(2, EVP_PKEY, "openssl.evp_pkey");
     lua_pushboolean(L, X509_check_private_key(cert, key));
@@ -372,8 +713,20 @@ static LUA_FUNCTION(openssl_x509_check)
   {
     X509_STORE* store = CHECK_OBJECT(2, X509_STORE, "openssl.x509_store");
     STACK_OF(X509)* untrustedchain = lua_isnoneornil(L, 3) ?  NULL : openssl_sk_x509_fromtable(L, 3);
-    int purpose = lua_isnone(L, 4) ? 0 : X509_PURPOSE_get_by_sname((char*)luaL_optstring(L, 4, "any"));
+    int purpose = 0;
     int ret = 0;
+    if (!lua_isnone(L, 4))
+    {
+      int purpose_id = X509_PURPOSE_get_by_sname((char*)luaL_optstring(L, 4, "any"));
+      if (purpose_id >= 0)
+      {
+        X509_PURPOSE* ppurpose = X509_PURPOSE_get0(purpose_id);
+        if (ppurpose)
+        {
+          purpose = ppurpose->purpose;
+        }
+      }
+    }
 #if 0
     X509_STORE_set_verify_cb_func(store, verify_cb);
 #endif
@@ -385,8 +738,73 @@ static LUA_FUNCTION(openssl_x509_check)
   }
 }
 
+#if OPENSSL_VERSION_NUMBER > 0x10002000L
+/***
+check x509 for host (only for openssl 1.0.2 or greater)
+@function check_host
+@tparam string host hostname to check for match match with x509 subject
+@treturn boolean result true if host is present and matches the certificate
+*/
+static LUA_FUNCTION(openssl_x509_check_host)
+{
+  X509 * cert = CHECK_OBJECT(1, X509, "openssl.x509");
+  if (lua_isstring(L, 2))
+  {
+    const char *hostname = lua_tostring(L, 2);
+    lua_pushboolean(L, X509_check_host(cert, hostname, strlen(hostname), 0, NULL));
+  }
+  else
+  {
+    lua_pushboolean(L, 0);
+  }
+  return 1;
+}
+/***
+check x509 for email address (only for openssl 1.0.2 or greater)
+@tparam string email to check for match match with x509 subject
+@treturn boolean result true if host is present and matches the certificate
+@function check_email
+*/
+static LUA_FUNCTION(openssl_x509_check_email)
+{
+  X509 * cert = CHECK_OBJECT(1, X509, "openssl.x509");
+  if (lua_isstring(L, 2))
+  {
+    const char *email = lua_tostring(L, 2);
+    lua_pushboolean(L, X509_check_email(cert, email, strlen(email), 0));
+  }
+  else
+  {
+    lua_pushboolean(L, 0);
+  }
+  return 1;
+}
+
+/***
+check x509 for ip address (ipv4 or ipv6, only for openssl 1.0.2 or greater)
+@function check_ip_asc
+@tparam string ip to check for match match with x509 subject
+@treturn boolean result true if host is present and matches the certificate
+*/
+static LUA_FUNCTION(openssl_x509_check_ip_asc)
+{
+  X509 * cert = CHECK_OBJECT(1, X509, "openssl.x509");
+  if (lua_isstring(L, 2))
+  {
+    const char *ip_asc = lua_tostring(L, 2);
+    lua_pushboolean(L, X509_check_ip_asc(cert, ip_asc, 0));
+  }
+  else
+  {
+    lua_pushboolean(L, 0);
+  }
+  return 1;
+}
+#endif
+
 IMP_LUA_SK(X509, x509)
 
+#if 0
 static STACK_OF(X509) * load_all_certs_from_file(BIO *in)
 {
   STACK_OF(X509) *stack = sk_X509_new_null();
@@ -414,7 +832,19 @@ static STACK_OF(X509) * load_all_certs_from_file(BIO *in)
   }
   return stack;
 };
+#endif
 
+/***
+get subject name of x509
+@function subject
+@treturn x509_name subject name
+*/
+/***
+set subject name of x509
+@function subject
+@tparam x509_name subject
+@treturn boolean result true for success
+*/
 static int openssl_x509_subject(lua_State* L)
 {
   X509* cert = CHECK_OBJECT(1, X509, "openssl.x509");
@@ -431,6 +861,19 @@ static int openssl_x509_subject(lua_State* L)
   }
 }
 
+/***
+get issuer name of x509
+@function issuer
+@tparam[opt=false] boolean asobject, true for return as x509_name object, or as table
+@treturn[1] x509_name issuer
+@treturn[1] table issuer name as table
+*/
+/***
+set issuer name of x509
+@function issuer
+@tparam x509_name name
+@treturn boolean result true for success
+*/
 static int openssl_x509_issuer(lua_State* L)
 {
   X509* cert = CHECK_OBJECT(1, X509, "openssl.x509");
@@ -447,13 +890,19 @@ static int openssl_x509_issuer(lua_State* L)
   }
 }
 
+/***
+get digest of x509 object
+@function digest
+@tparam[opt='sha1'] evp_digest|string md_alg, default use 'sha1'
+@treturn string digest result
+*/
 static int openssl_x509_digest(lua_State* L)
 {
   unsigned int bytes;
   unsigned char buffer[EVP_MAX_MD_SIZE];
   char hex_buffer[EVP_MAX_MD_SIZE * 2];
   X509 *cert = CHECK_OBJECT(1, X509, "openssl.x509");
-  const EVP_MD *digest = lua_isnoneornil(L, 2) ? EVP_sha1() : get_digest(L, 2);
+  const EVP_MD *digest = get_digest(L, 2, "sha256");
   int ret;
   if (!digest)
   {
@@ -471,6 +920,16 @@ static int openssl_x509_digest(lua_State* L)
   return openssl_pushresult(L, ret);
 };
 
+/***
+get notbefore valid time of x509
+@function notbefore
+@treturn string notbefore time string
+*/
+/***
+set notbefore valid time of x509
+@function notbefore
+@tparam string|number notbefore
+*/
 static int openssl_x509_notbefore(lua_State *L)
 {
   X509* cert = CHECK_OBJECT(1, X509, "openssl.x509");
@@ -488,7 +947,7 @@ static int openssl_x509_notbefore(lua_State *L)
       at = ASN1_TIME_new();
       ASN1_TIME_set(at, time);
     }
-    if (lua_isstring(L, 2))
+    else if (lua_isstring(L, 2))
     {
       const char* time = lua_tostring(L, 2);
       at = ASN1_TIME_new();
@@ -501,6 +960,7 @@ static int openssl_x509_notbefore(lua_State *L)
     if (at)
     {
       ret = X509_set_notBefore(cert, at);
+      ASN1_TIME_free(at);
     }
     else
       ret = 0;
@@ -508,6 +968,16 @@ static int openssl_x509_notbefore(lua_State *L)
   };
 }
 
+/***
+get notafter valid time of x509
+@function notafter
+@treturn string notafter time string
+*/
+/***
+set notafter valid time of x509
+@function notafter
+@tparam string|number notafter
+*/
 static int openssl_x509_notafter(lua_State *L)
 {
   X509* cert = CHECK_OBJECT(1, X509, "openssl.x509");
@@ -525,7 +995,7 @@ static int openssl_x509_notafter(lua_State *L)
       at = ASN1_TIME_new();
       ASN1_TIME_set(at, time);
     }
-    if (lua_isstring(L, 2))
+    else if (lua_isstring(L, 2))
     {
       const char* time = lua_tostring(L, 2);
       at = ASN1_TIME_new();
@@ -538,6 +1008,7 @@ static int openssl_x509_notafter(lua_State *L)
     if (at)
     {
       ret = X509_set_notAfter(cert, at);
+      ASN1_TIME_free(at);
     }
     else
       ret = 0;
@@ -545,6 +1016,21 @@ static int openssl_x509_notafter(lua_State *L)
   }
 }
 
+/***
+check x509 valid
+@function validat
+@tparam[opt] number time, default will use now time
+@treturn boolean result true for valid, or for invalid
+@treturn string notbefore
+@treturn string notafter
+*/
+/***
+set valid time, notbefore and notafter
+@function validat
+@tparam number notbefore
+@tparam number notafter
+@treturn boolean result, true for success
+*/
 static int openssl_x509_valid_at(lua_State* L)
 {
   X509* cert = CHECK_OBJECT(1, X509, "openssl.x509");
@@ -592,6 +1078,19 @@ static int openssl_x509_valid_at(lua_State* L)
   return 0;
 }
 
+/***
+get serial number of x509
+@function serial
+@tparam[opt=true] boolean asobject
+@treturn[1] bn object
+@treturn[2] string result
+*/
+/***
+set serial number of x509
+@function serial
+@tparam string|number|bn serail
+@treturn boolean result true for success
+*/
 static int openssl_x509_serial(lua_State *L)
 {
   X509* cert = CHECK_OBJECT(1, X509, "openssl.x509");
@@ -620,7 +1119,7 @@ static int openssl_x509_serial(lua_State *L)
   else
   {
     int ret;
-    if (auxiliar_isclass(L, "openssl.asn1_string", 2))
+    if (auxiliar_getclassudata(L, "openssl.asn1_string", 2))
     {
       serial = CHECK_OBJECT(2, ASN1_STRING, "openssl.asn1_string");
     }
@@ -638,6 +1137,17 @@ static int openssl_x509_serial(lua_State *L)
   return 1;
 }
 
+/***
+get version number of x509
+@function version
+@treturn number version of x509
+*/
+/***
+set version number of x509
+@function version
+@tparam number version
+@treturn boolean result true for result
+*/
 static int openssl_x509_version(lua_State *L)
 {
   int version;
@@ -657,12 +1167,25 @@ static int openssl_x509_version(lua_State *L)
   }
 }
 
+/***
+get extensions of x509 object
+@function extensions
+@tparam[opt=false] boolean asobject, true for return as stack_of_x509_extension or as table
+@treturn[1] stack_of_x509_extension object when param set true
+@treturn[2] table contain all x509_extension when param set false or nothing
+*/
+/***
+set extension of x509 object
+@function extensions
+@tparam stack_of_x509_extension extensions
+@treturn boolean result true for success
+*/
 static int openssl_x509_extensions(lua_State* L)
 {
-  X509 *peer = CHECK_OBJECT(1, X509, "openssl.x509");
+  X509 *self = CHECK_OBJECT(1, X509, "openssl.x509");
+  STACK_OF(X509_EXTENSION) *exts = (STACK_OF(X509_EXTENSION) *)X509_get0_extensions(self);
   if (lua_isnone(L, 2))
   {
-    STACK_OF(X509_EXTENSION) *exts = peer->cert_info->extensions;
     if (exts)
     {
       openssl_sk_x509_extension_totable(L, exts);
@@ -673,89 +1196,45 @@ static int openssl_x509_extensions(lua_State* L)
   }
   else
   {
-    STACK_OF(X509_EXTENSION) *exts = openssl_sk_x509_extension_fromtable(L, 2);
-    sk_X509_EXTENSION_pop_free(peer->cert_info->extensions, X509_EXTENSION_free);
-    peer->cert_info->extensions = exts;
+    STACK_OF(X509_EXTENSION) *others = (STACK_OF(X509_EXTENSION) *)openssl_sk_x509_extension_fromtable(L, 2);
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
+    sk_X509_EXTENSION_pop_free(self->cert_info->extensions, X509_EXTENSION_free);
+    self->cert_info->extensions = others;
+#else
+    int i;
+    int n = sk_X509_EXTENSION_num(exts);
+    for (i = 0; i < n; i++)
+      sk_X509_EXTENSION_delete(exts, i);
+    n = sk_X509_EXTENSION_num(others);
+    for (i = 0; i < n; i++)
+    {
+      X509_EXTENSION* ext = sk_X509_EXTENSION_value(others, i);
+      if (exts!=NULL)
+        sk_X509_EXTENSION_push(exts, ext);
+      else
+        X509_add_ext(self, ext, -1);
+    }
+    sk_X509_EXTENSION_free(others);
+#endif
     return openssl_pushresult(L, 1);
   }
 }
 
-static int openssl_x509_new(lua_State* L)
-{
-  int i = 1;
-  int ret = 1;
-  int n = lua_gettop(L);
-  X509 *x = X509_new();
-
-  ret = X509_set_version(x, 2);
-  if (ret == 1 && (
-        auxiliar_isclass(L, "openssl.bn", i) ||
-        lua_isstring(L, i) || lua_isnumber(L, i)
-      ))
-  {
-    BIGNUM *bn = BN_get(L, i);
-    ASN1_INTEGER* ai = BN_to_ASN1_INTEGER(bn, NULL);
-    BN_free(bn);
-    ret = X509_set_serialNumber(x, ai);
-    ASN1_INTEGER_free(ai);
-    i++;
-  }
-
-  for (; i <= n; i++)
-  {
-    if (ret == 1 && auxiliar_isclass(L, "openssl.x509_req", i))
-    {
-      X509_REQ* csr = CHECK_OBJECT(i, X509_REQ, "openssl.x509_req");
-      X509_NAME* xn = X509_REQ_get_subject_name(csr);
-      ret = X509_set_subject_name(x, xn);
-
-      if (ret == 1)
-      {
-        STACK_OF(X509_EXTENSION) *exts = X509_REQ_get_extensions(csr);
-        int j, n1;
-        n1 = sk_X509_EXTENSION_num(exts);
-        for (j = 0; ret == 1 && j < n1; j++)
-        {
-          ret = X509_add_ext(x, sk_X509_EXTENSION_value(exts, j), j);
-        }
-        sk_X509_EXTENSION_pop_free(exts, X509_EXTENSION_free);
-      }
-      if (ret == 1)
-      {
-        EVP_PKEY* pkey = X509_REQ_get_pubkey(csr);
-        ret = X509_set_pubkey(x, pkey);
-        EVP_PKEY_free(pkey);
-      }
-      i++;
-    };
-
-    if (ret == 1 && auxiliar_isclass(L, "openssl.x509_name", i))
-    {
-      X509_NAME *xn = CHECK_OBJECT(i, X509_NAME, "openssl.x509_name");
-      ret = X509_set_subject_name(x, xn);
-      i++;
-    }
-  }
-
-  if (ret == 1)
-  {
-    PUSH_OBJECT(x, "openssl.x509");
-    return 1;
-  }
-  else
-  {
-    X509_free(x);
-    return openssl_pushresult(L, ret);
-  }
-};
-
+/***
+sign x509
+@function sign
+@tparam evp_pkey pkey private key to sign x509
+@tparam x509|x509_name cacert or cacert x509_name
+@tparam[opt='sha1WithRSAEncryption'] string|md_digest md_alg
+@treturn boolean result true for check pass
+*/
 static int openssl_x509_sign(lua_State*L)
 {
   X509* x = CHECK_OBJECT(1, X509, "openssl.x509");
-  if (lua_isnoneornil(L, 2) && x->cert_info != NULL)
+  if (lua_isnoneornil(L, 2))
   {
     unsigned char *out = NULL;
-    int len = i2d_X509_CINF(x->cert_info, &out);
+    int len = i2d_re_X509_tbs(x, &out);
     if (len > 0)
     {
       lua_pushlstring(L, (const char *)out, len);
@@ -764,13 +1243,13 @@ static int openssl_x509_sign(lua_State*L)
     }
     return openssl_pushresult(L, len);
   }
-  else if (auxiliar_isclass(L, "openssl.evp_pkey", 2))
+  else if (auxiliar_getclassudata(L, "openssl.evp_pkey", 2))
   {
     EVP_PKEY* pkey = CHECK_OBJECT(2, EVP_PKEY, "openssl.evp_pkey");
     const EVP_MD *md;
     int ret = 1;
     int i = 3;
-    if (auxiliar_isclass(L, "openssl.x509_name", 3))
+    if (auxiliar_getclassudata(L, "openssl.x509_name", 3))
     {
       X509_NAME* xn = CHECK_OBJECT(3, X509_NAME, "openssl.x509_name");
       ret = X509_set_issuer_name(x, xn);
@@ -790,9 +1269,7 @@ static int openssl_x509_sign(lua_State*L)
 
     if (ret == 1)
     {
-      md = lua_isnoneornil(L, i) ?
-           EVP_get_digestbyname("sha1") :
-           get_digest(L, i);
+      md = get_digest(L, i, "sha256");
       ret = X509_sign(x, pkey, md);
       if (ret > 0)
         ret = 1;
@@ -804,11 +1281,16 @@ static int openssl_x509_sign(lua_State*L)
     size_t sig_len;
     const char* sig = luaL_checklstring(L, 2, &sig_len);
     int nid = openssl_get_nid(L, 3);
-    int ret = ASN1_BIT_STRING_set(x->signature, (unsigned char*)sig, (int)sig_len);
+    CONSTIFY_X509_get0 ASN1_BIT_STRING *psig = NULL;
+    CONSTIFY_X509_get0 X509_ALGOR *palg = NULL;
+    int ret;
+
+    X509_get0_signature(&psig, &palg, x);
+    ret = ASN1_BIT_STRING_set((ASN1_BIT_STRING*)psig, (unsigned char*)sig, (int)sig_len);
     if (ret == 1)
     {
       ASN1_OBJECT *obj = OBJ_nid2obj(nid);
-      ret = X509_ALGOR_set0(x->sig_alg, obj, V_ASN1_UNDEF, NULL);
+      ret = X509_ALGOR_set0((X509_ALGOR*)palg, obj, V_ASN1_UNDEF, NULL);
     }
     return openssl_pushresult(L, ret);
   }
@@ -817,22 +1299,31 @@ static int openssl_x509_sign(lua_State*L)
 static int openssl_x509_verify(lua_State*L)
 {
   X509* x = CHECK_OBJECT(1, X509, "openssl.x509");
-  if (lua_isnoneornil(L, 2) && x->cert_info != NULL)
+  if (lua_isnoneornil(L, 2))
   {
     unsigned char *out = NULL;
-    int len = i2d_X509_CINF(x->cert_info, &out);
+    int len = i2d_re_X509_tbs(x, &out);
     if (len > 0)
     {
+      CONSTIFY_X509_get0 ASN1_BIT_STRING *psig = NULL;
+      CONSTIFY_X509_get0 X509_ALGOR *palg = NULL;
+
       lua_pushlstring(L, (const char *)out, len);
       OPENSSL_free(out);
-      if (x->signature != NULL)
+
+      X509_get0_signature(&psig, &palg, x);
+      if (psig != NULL)
       {
-        lua_pushlstring(L, (const char *)x->signature->data, x->signature->length);
+        lua_pushlstring(L, (const char *)psig->data, psig->length);
       }
       else
         lua_pushnil(L);
-      if (x->sig_alg)
-        openssl_push_x509_algor(L, x->sig_alg);
+
+      if (palg)
+      {
+        X509_ALGOR *alg = X509_ALGOR_dup((X509_ALGOR *)palg);
+        PUSH_OBJECT(alg, "openssl.x509_algor");
+      }
       else
         lua_pushnil(L);
       return 3;
@@ -852,6 +1343,11 @@ static luaL_Reg x509_funcs[] =
   {"parse",       openssl_x509_parse},
   {"export",      openssl_x509_export},
   {"check",       openssl_x509_check},
+#if OPENSSL_VERSION_NUMBER > 0x10002000L
+  {"check_host",  openssl_x509_check_host},
+  {"check_email", openssl_x509_check_email},
+  {"check_ip_asc", openssl_x509_check_ip_asc},
+#endif
   {"pubkey",      openssl_x509_public_key},
   {"version",     openssl_x509_version},
 
@@ -874,181 +1370,6 @@ static luaL_Reg x509_funcs[] =
   {NULL,      NULL},
 };
 
-
-static int openssl_push_purpose(lua_State*L , X509_PURPOSE* purpose)
-{
-  lua_newtable(L);
-
-  AUXILIAR_SET(L, -1, "purpose", purpose->purpose, integer);
-  AUXILIAR_SET(L, -1, "trust", purpose->trust, integer);
-  AUXILIAR_SET(L, -1, "flags", purpose->flags, integer);
-
-  AUXILIAR_SET(L, -1, "name", purpose->name, string);
-  AUXILIAR_SET(L, -1, "sname", purpose->sname, string);
-
-  return 1;
-};
-
-static int openssl_x509_purpose(lua_State*L)
-{
-  if (lua_isnoneornil(L, 1))
-  {
-    int count = X509_PURPOSE_get_count();
-    int i;
-    lua_newtable(L);
-    for (i = 0; i < count; i++)
-    {
-      X509_PURPOSE* purpose = X509_PURPOSE_get0(i);
-      openssl_push_purpose(L, purpose);
-      lua_rawseti(L, -2, i + 1);
-    }
-    return 1;
-  }
-  else if (lua_isnumber(L, 1))
-  {
-    int idx = X509_PURPOSE_get_by_id(lua_tointeger(L, 1));
-    if (idx >= 0)
-    {
-      X509_PURPOSE* purpose = X509_PURPOSE_get0(idx);
-      openssl_push_purpose(L, purpose);
-    }
-    else
-      lua_pushnil(L);
-    return 1;
-  }
-  else if (lua_isstring(L, 1))
-  {
-    char* name = (char*)lua_tostring(L, 1);
-    int idx = X509_PURPOSE_get_by_sname(name);
-    if (idx >= 0)
-    {
-      X509_PURPOSE* purpose = X509_PURPOSE_get0(idx);
-      openssl_push_purpose(L, purpose);
-    }
-    else
-      lua_pushnil(L);
-    return 1;
-  }
-  return 0;
-};
-
-static const char* usage_mode[] =
-{
-  "standard",
-  "netscape",
-  "extend",
-  NULL
-};
-
-static int openssl_x509_certtypes(lua_State*L)
-{
-  int mode = luaL_checkoption(L, 1, "standard", usage_mode);
-  int i;
-  const BIT_STRING_BITNAME* bitname;
-
-  switch (mode)
-  {
-  case 0:
-  {
-    const static BIT_STRING_BITNAME key_usage_type_table[] =
-    {
-      {0, "Digital Signature", "digitalSignature"},
-      {1, "Non Repudiation", "nonRepudiation"},
-      {2, "Key Encipherment", "keyEncipherment"},
-      {3, "Data Encipherment", "dataEncipherment"},
-      {4, "Key Agreement", "keyAgreement"},
-      {5, "Certificate Sign", "keyCertSign"},
-      {6, "CRL Sign", "cRLSign"},
-      {7, "Encipher Only", "encipherOnly"},
-      {8, "Decipher Only", "decipherOnly"},
-      { -1, NULL, NULL}
-    };
-    lua_newtable(L);
-    for (i = 0, bitname = &key_usage_type_table[i]; bitname->bitnum != -1; i++, bitname = &key_usage_type_table[i])
-    {
-      openssl_push_bit_string_bitname(L, bitname);
-      lua_rawseti(L, -2, i + 1);
-    }
-    return 1;
-
-  }
-  case 1:
-  {
-    const static BIT_STRING_BITNAME ns_cert_type_table[] =
-    {
-      {0, "SSL Client", "client"},
-      {1, "SSL Server", "server"},
-      {2, "S/MIME", "email"},
-      {3, "Object Signing", "objsign"},
-      {4, "Unused", "reserved"},
-      {5, "SSL CA", "sslCA"},
-      {6, "S/MIME CA", "emailCA"},
-      {7, "Object Signing CA", "objCA"},
-      { -1, NULL, NULL}
-    };
-    lua_newtable(L);
-    for (i = 0, bitname = &ns_cert_type_table[i]; bitname->bitnum != -1; i++, bitname = &ns_cert_type_table[i])
-    {
-      openssl_push_bit_string_bitname(L, bitname);
-      lua_rawseti(L, -2, i + 1);
-    }
-    return 1;
-  }
-  case 2:
-  {
-    static const int ext_nids[] =
-    {
-      NID_server_auth,
-      NID_client_auth,
-      NID_email_protect,
-      NID_code_sign,
-      NID_ms_sgc,
-      NID_ns_sgc,
-      NID_OCSP_sign,
-      NID_time_stamp,
-      NID_dvcs,
-      NID_anyExtendedKeyUsage
-    };
-    int count = sizeof(ext_nids) / sizeof(int);
-    int nid;
-    lua_newtable(L);
-    for (i = 0; i < count; i++)
-    {
-      nid = ext_nids[i];
-      lua_newtable(L);
-      lua_pushstring(L, OBJ_nid2ln(nid));
-      lua_setfield(L, -2, "lname");
-      lua_pushstring(L, OBJ_nid2sn(nid));
-      lua_setfield(L, -2, "sname");
-      lua_pushinteger(L, nid);
-      lua_setfield(L, -2, "nid");
-      lua_rawseti(L, -2, i + 1);
-    };
-    return 1;
-  }
-  }
-  return 0;
-}
-
-static int openssl_verify_cert_error_string(lua_State*L)
-{
-  int v = luaL_checkint(L, 1);
-  const char*s = X509_verify_cert_error_string(v);
-  lua_pushstring(L, s);
-  return 1;
-}
-
-static luaL_Reg R[] =
-{
-  {"new",           openssl_x509_new },
-  {"read",          openssl_x509_read },
-  {"purpose",       openssl_x509_purpose },
-  {"certtypes",     openssl_x509_certtypes },
-  {"verify_cert_error_string", openssl_verify_cert_error_string },
-
-  {NULL,    NULL}
-};
-
 int luaopen_x509(lua_State *L)
 {
   auxiliar_newclass(L, "openssl.x509", x509_funcs);
