@@ -3,8 +3,7 @@ EXPORT_ASSERT_TO_GLOBALS = true
 require'luaunit'
 
 openssl.rand_load()
-v = {openssl.version(true)}
-print(openssl.version())
+print('VERSION:', openssl.version())
 
 dofile('0.engine.lua')
 dofile('0.misc.lua')
@@ -23,11 +22,15 @@ dofile('5.x509_crl.lua')
 dofile('5.x509.lua')
 dofile('5.ts.lua')
 dofile('6.pkcs7.lua')
+dofile('6.cms.lua')
 dofile('7.pkcs12.lua')
 dofile('8.ssl_options.lua')
 dofile('8.ssl.lua')
+dofile('9.srp.lua')
+dofile('9.issue.lua')
 dofile('rsa.lua')
 dofile('ec.lua')
+dofile('sm2.lua')
 
 --LuaUnit.verbosity = 0
 LuaUnit.run()
